# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import json
import path
import computational_job_input_action
import logging
import jsonschema

class ComputationalJobInputActionCreateJSONFile(computational_job_input_action.ComputationalJobInputAction):

    def __init__(self, input_ref, json_filepath, json_path_expr=None, json_schema=None):
        super(ComputationalJobInputActionCreateJSONFile, self).__init__()
        self._input_ref = input_ref
        self._json_filepath = json_filepath if isinstance(json_filepath, path.path) else path.path(json_filepath)

        # Python-like expression to access json data in memory.
        # This value is used in case of multiple json file creation to enumerate the file names (e.g. outfile1.json, outfile2.json) using an id from the input json data
        self._json_path_expr = json_path_expr
        self.json_files_created = []

        if isinstance(json_schema, path.path):
            self._json_schema = json.loads(json_schema.text())
        else:
            self._json_schema = json.loads(json_schema)


    def set_inputs(self, inputs):
        #for par_name in self._input_refs:
        logger = logging.getLogger("ComputationalJobInputActionCreateJSONFile.set_inputs")
        logger.debug("start creating JSON file")
        if self._input_ref in inputs.names():
            text2write = inputs[self._input_ref].get_value()
            if type(text2write) == list:
                logger.debug("a list of asset is given, iterate over json texts")
                for single_json_text in text2write:
                    if self.validate_json(single_json_text):
                        self.create_json_file(single_json_text)
            elif type(text2write) != list:
                logger.debug("only one json string is give, just write the file with the value of the json path expr")
                if self.validate_json(text2write):
                    self.create_json_file(text2write)
            else:
                msg="Cannot produce JSON file with current configuration and input in ComputationalJobInputActionCreateJSONFile"
                logger.debug(msg)
                raise Exception(msg)

    def validate_json(self, json_text):
        logger = logging.getLogger("ComputationalJobInputActionCreateJSONFile.validate_json")
        if self._json_schema == None:
            logger.warning("cannot validate json input against json schema")
            return True
        try:
            jsonschema.validate(json_text, self._json_schema)
            return True
        except Exception as ex:
            msg = "Invalid json for asset: " + str(ex)
            logger.fatal(msg)
            raise ex

    def create_json_file(self, json_text):
        json_filepath = self._extract_id_from_json(json_text)
        #json_filepath.write_text( json_text )
        fp = json_filepath.open('w')
        json.dump( json_text, fp )
        fp.close()
        self.json_files_created.append( json_filepath )


    def _extract_id_from_json(self, json_text):
        if self._json_path_expr != None and self._json_path_expr != '':
            _id = eval( "json_text" + self._json_path_expr )
            self._json_filepath = str(self._json_filepath).replace('${json_path_expr}', '%i')
            fp = self._json_filepath % _id
        else:
            fp = self._json_filepath
        return path.path(fp)

    def exists(self):
        files_exists = map(lambda fp: fp.exists(),  self.json_files_created)
        files_exists = list(set(files_exists))
        return len(files_exists)==1 and files_exists[0]



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

class ComputationalJobInputAction(object):

    def __init__(self):
        pass

    def can_produce_cmd_line(self):
        m = getattr(self, "get_cmd_line", None)
        return callable(m)



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import copy
import computational_job_input_action

class ComputationalJobInputActionCmdParam(computational_job_input_action.ComputationalJobInputAction): #computational_job_input_action.ComputationalJobInputAction

    def __init__(self, input_ref, template="--name=value", alias=None):
        super(ComputationalJobInputActionCmdParam, self).__init__()
        #if not type(input_refs) is list:
        #    self._input_refs = [input_refs]
        #else:
        self._input_ref = input_ref
        self._template = template
        self._cmdline = ""
        self._alias = alias

    def set_inputs(self, inputs):
        #for varname in self._input_ref:
        if self._input_ref in inputs.names():
            self._cmdline += ' ' + self._instance_template( self._input_ref, inputs[self._input_ref].get_value_string() ) 
        self._cmdline = self._cmdline.strip()

    def _instance_template(self, name, value_str):
        cmd=''
        if self._alias!=None:
            name = self._alias
        hasName = False
        hasValue = False
        template = copy.deepcopy(self._template)
        if 'name' in template:
            template = template.replace("name", "%s")
            hasName = True
        if 'value' in self._template:
            template =template.replace("value","%s")  
            hasValue = True

        if (hasName and hasValue):
            cmd += template % (name, value_str) 
        elif hasName:
            cmd += template % (name)
        elif hasValue:
            cmd += template % (value_str)
        else:
            raise Exception("Bad template for command line parameter " + name)

        return cmd 

    def get_cmd_line(self):
        return self._cmdline




# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import string
import datetime
import json

class InputParameter(object):

    def __init__(self, name):
        self._name=name
        self._alias=None
        self._type=None
        self._title=None
        self._description = None
        self._min=0
        self._max=1
        self._default = None
        self._formatter = None
        self._input_mime_type = None
        self._allowed_chars = string.printable.replace('-','').replace(' ','')

        self._value=None

    def inject_values(self, paremeters_types_defs):
        #override defaults with actual values
        for k,v in paremeters_types_defs.items():
            if hasattr(self, "_" + k):
                setattr(self, "_" + k, v)

        self._min=int(self._min)
        self._max=int(self._max)

    def set_actual_value(self, value):
        if type(value) is not list:
            self._value = [value]
        else:
            self._value = value

    def has_value(self):    
        return self._value <> None

    def get_value(self):
        return self._value

    def _type_checking(self, value):
        try:
            if (self._type=='int'):
                int(value)
                return self._type
            elif (self._type=='float'):
                float(value)
                return self._type
            elif (self._type=='string'):
                return all(c in  self._allowed_chars for c in value)
            elif (self._type=='datetime' and self._formatter!=None):
                #to check
                datetime.datetime.strptime(value, self._formatter)
                return self._type
            elif (self._type=='application/json'):
                json.loads(value)
                return self._type
            else:
                return False        
        except:
            return False

    def validate(self):
        if (not self.has_value()):
            raise Exception("cannot find a value for parameter " + self.get_name())

        if not (self._min <= len(self._value) and len(self._value) <= self._max):
            raise Exception("Actual value for parameter has wrong multiplicity")

        res = map(self._type_checking, self._value)
        #remove duplicates
        res = list(set(res))

        #if res has one element equal to True means all items in the list passed type checking
        if (len(res)==1 and res[0]):
            return True
        else:
            raise Exception("Bad type")

    def get_name(self):
        return self._alias if self._alias <> None else self._name

    def get_input_mime_type(self):
        return self._input_mime_type

    def get_name_no_alias(self):
        return self._name

    def get_cmd_line(self):
        pass

    def as_json_string(self):
        #{"type": "string", "description": "A persons surname", "max": 1, "default": "Meier"}
        res={}
        attrib_to_convert = ['_type', '_title', '_description', '_min', '_max', '_default', '_input_mime_type']
        attribute_list = [a for a in dir(self) if not a.startswith('__') and not callable(getattr(self, a))]
        attribute_list_filtered = [x for x in attribute_list if x in attrib_to_convert]
        for a in attribute_list_filtered:
            res[a[1:]] = getattr(self, a)
        return json.dumps(res)



from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA
import base64
import sys

def encrypt(message, externKey, passphrase):
    publickey = open(externKey, "r")
    rsa_key = RSA.importKey(publickey, passphrase=passphrase)
    # verifier = PKCS1_v1_5.new(rsa_key)
    # h = SHA.new(message)
    encriptedData = rsa_key.encrypt(message, 0)
    print(base64.b64encode(encriptedData[0]))

if __name__ == "__main__":
    if len(sys.argv) == 4:
        encrypt(sys.argv[1], sys.argv[2], sys.argv[3])
    else:
        print("Usage: python encrypt.py password path/to/rsakey.pub passphrase")



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import introspection
from collections import OrderedDict

import command_line_parameter

class InputParameters(object):

        @staticmethod
        def create_from_config(input_sections):
            '''Create a InputParameters object.

            input_sections: a dictionary such as { input1 : [( 'par1_input1_name' , par1_input1_value ), ( 'par2_input1_name' , par2_input1_value ), ...], input2 : [ .... ], ... }
            '''
            input_sections_reshaped = OrderedDict()
            #force the order of sections in config files: [input1], [input2], etc
            for k in sorted(input_sections):
                d=dict(input_sections[k])
                name = d['name']
                del d['name']
                input_sections_reshaped[name] = d
            return InputParameters( input_sections_reshaped )

        def __init__(self, paremeters_types_defs):
            '''Create a input parameters set from a definitions

            paremeters_types_defs is a dictionary such as {par1_name : { 'par1_attrubute1_name' : par1_attrubute1_value, ... }, par2_name : { 'par2_attrubute1_name' : par2_attrubute1_value, ... }, ... }
            '''
            self._params = OrderedDict()

            for name, d in OrderedDict(paremeters_types_defs).items():
                if ('class' in d):
                    try:
                        self._params[name] = introspection.get_class_one_arg( d['class'], name )
                    except:
                        raise #log("Cannot create parameter object from class name "+d['class']+" for parameter " + name)
                else:
                    self._params[name] = command_line_parameter.CommandLineParameter(name)
                self._params[name].inject_values( d ) 

        def parse(self, input_variables=None):
            """if input_variables is None all params are of type CommandLineParameterConst and input data is read from config file"""
            for n,v in input_variables.items():
                if n in self._params.keys():
                    self._params[n].set_actual_value(v)
                else:
                    pass #todo: log "received unknown input parameter " + n 
            self.validate()

        def validate(self):
            for k in self._params.keys():
                self._params[k].validate()  

        def get_cmd_line(self):
            cmd_line=''
            for k in self._params.keys():
                c = self._params[k].get_cmd_line()
                if c <> None:
                    cmd_line +=  c 
            return cmd_line

        def as_DLR_protocol(self):
            #[('result', '{"type": "string", ...ut.xml" }')]
            res = []
            for k in self._params.keys():
                n = self._params[k].get_name_no_alias()
                j = self._params[k].as_json_string()
                res.append( (n,j) )
            return res
        
        def checkForCodeInsertion(self, argList):
            ''' Check user input for bad code insertion
            
                @param argList: user arguments
                @return: False if bad code found, else True
            '''
            # Check command line insertions
            maliciousCommands = ['>', '<', '>>', '|', '>&', '<&']
            # Check for bad code insertion
            maliciousCode = ['eval(', 'exec(', 'execfile(', 'input(']

            # Evaluate malicious commands
            if any(e in maliciousCommands for e in argList):
                raise IOError('Found bad code in user input')
            # Evaluate maliciousCode
            for element in argList:
                for code in maliciousCode:
                    if code in element:
                        raise IOError('Found bad code in user input')



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import os

class FileLikeObjectMock(object):
    def __init__(self, lines, linesep="\n"):
        if type(lines) is list:
            self._lines = lines
        else:
            self._lines = lines.split(linesep)
            self._lines = map(lambda l : l.strip(), self._lines)
        self._lp=0
    
    def readline(self):
        if self._lp-1 >= len(self._lines):
            return ''
        else:
            self._lp += 1
            return self._lines[self._lp-1]





# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

import subprocess
import logging.config
import logging
import argparse
import sys
import thread
import traceback
import logging
import sys
import os
import uuid
import zipfile

# constants
#id = os.urandom(10)
id = str(uuid.uuid4())
gdalContour = r'/usr/bin/gdal_contour'
dst = r'contour_'+id[:13]
src = '%s/../../../resource_dir/srtm_39_04/srtm_39_04_c.tif' % os.path.dirname(os.path.abspath(__file__))
cmd = '-a elev'  # just for example!
interval = '-i'

class GDALTest(object):
    def __init__(self, args):
        self.args=args
        self.create_logger("logger_test.properties")
        self.logger.info("ProgressInfo:0.0%")


    def run(self):
        trg = '%s/../../../output/%s/%s.shp' % (os.path.dirname(os.path.abspath(__file__)), self.args.execution_id, dst)

        #fullCmd = ' '.join([gdalContour, cmd, self.youCanQuoteMe(src), self.youCanQuoteMe(dst), interval, self.args.interval])
        fullCmd = ' '.join([gdalContour, cmd, src, trg, interval, self.args.interval])
        self.logger.debug("Running command > " + fullCmd)
        proc=subprocess.Popen(fullCmd.split(), stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=False)
        for line in proc.stdout:
            self.logger.info(line)

        #call communicate to retrieve return code of subprocess
        proc.communicate()
        ret = proc.returncode

        if (ret == 0):
            # zipf = zipfile.ZipFile(self.args.workdir+'/contour.zip', 'w')
            # self.zipdir(self.args.workdir+'/', zipf)
            output_dir = '%s/../../../output/%s' % (os.path.dirname(os.path.abspath(__file__)), self.args.execution_id)
            zipf = zipfile.ZipFile(output_dir+'/contour.zip', 'w')
            self.zipdir(output_dir+'/', zipf)
            zipf.close()

            self.logger.info("ProgressInfo:100%")
        else:
            self.logger.critical("Error occurred during processing.")

        return ret
    # see note below
    def youCanQuoteMe(self, item):
        return "\"" + item + "\""

    def zipdir(self, path, zip):
        for root, dirs, files in os.walk(path):
            files = [ fi for fi in files if fi.startswith(dst) ]
            for file in files:
                zip.write(os.path.join(root, file))

    def create_logger(self, logger_config_file):
        defaults={}

        logging.config.fileConfig(str(logger_config_file),  defaults=defaults)

        self.logger = logging.getLogger("main.create_logger")

        self.logger.debug("Logger initialized with file " + str(logger_config_file))

if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--interval", nargs='?', default="10", help="Elevation interval between contours.")
    parser.add_argument("-w", "--workdir", nargs='?', default="", help="Remote process sandbox working directory.")
    parser.add_argument("-e", "--execution_id", nargs='?', default="", help="Remote process Unique Execution Id.")
    cmdargs = parser.parse_args()

    gdalTest = GDALTest(cmdargs)
    return_code = gdalTest.run()
    sys.exit(return_code)



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import ConfigParser
import path
import types
from string import strip

def create(filePath, moreSerchPath=[], variables={}, raw=False, case_sensitive=False):

    config = ConfigParser.ConfigParser(allow_no_value=raw, defaults=variables)

    #force config lookup paramters to be to case sensitive
    if case_sensitive:
        config.optionxform = lambda option: option
    fp=open(filePath)
    config.readfp( fp  )
    fp.close()

    config.items_without_defaults = types.MethodType(items_without_defaults,config)
    config.get_list = types.MethodType(get_list_impl, config)
    config.get_list_list = types.MethodType(get_list_list_impl, config)
    config.get_list_int = types.MethodType(get_list_int_impl, config)
    config.get_list_float = types.MethodType(get_list_float_impl, config)
    config.get_password =  types.MethodType(get_password, config)
    config.get_path =  types.MethodType(get_path, config)
    config.get_list_path = types.MethodType(get_list_path_impl, config)
    return config

def items_without_defaults(target, section_name, raw=False):
    #return [s for s in target.items(section_name, raw=raw) if not s[0] in target.defaults().keys()]
    res=[]
    default_section_items = dict(target.items('DEFAULT', raw=raw))
    for s in target.items(section_name, raw=raw) :
        if s[0] in target.defaults().keys(): #same name in defaults+section_name and defaults
            if s[1] == default_section_items[s[0]]: #same value defaults+section_name and defaults
                pass
            else:
                res.append(s)
        else:
            res.append(s)
    return res


def get_list_impl(target, section, itemname):
    v=target.get(section, itemname)
    if (v==None or v==''):
        return None
    v = v.strip()
    v = v[1:-1]
    t = v.split(",")
    return map(strip, t)

def get_list_list_impl(target, section, itemname):
    l1 = get_list_impl(target, section, itemname)
    res=[]
    for el in l1:
        tmp=el.split("|")
        res.append ( map(strip, tmp) )
    return res


def get_list_int_impl(target, section, itemname):
    v=target.get(section, itemname)
    if (v==None or v==''):
        return None
    v = v.strip()
    v = v[1:-1]
    t = v.split(",")
    res=map(strip, t)
    return map(int,res)

def get_list_float_impl(target, section, itemname):
    v=target.get(section, itemname)
    if (v==None or v==''):
        return None
    v = v.strip()
    v = v[1:-1]
    t = v.split(",")
    res=map(strip, t)
    return map(float,res)

def get_list_path_impl(target, section, itemname):
    v=target.get(section, itemname)
    if (v==None or v==''):
        return None
    v = v.strip()
    v = v[1:-1]
    t = v.split(",")
    res=map(strip, t)
    return map(path.path,res)

def get_password(target, section, itemname):
    psw=target.get(section, itemname)
    if (psw==None):
        return None
    if psw=="ask":
        import getpass
        psw = getpass.getpass("insert password:")
    return psw

def get_path(target, section, itemname):
    p=target.get(section, itemname)
    if (p==None):
        return None
    return path.path(p)



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import path
import computational_job_input_action

class ComputationalJobInputActionCopyFile(computational_job_input_action.ComputationalJobInputAction):

    def __init__(self, source, target):
        super(ComputationalJobInputActionCopyFile, self).__init__()

        self._source = source if isinstance(source, path.path) else path.path(source)
        self._target = target if isinstance(target, path.path) else path.path(target)

    def set_inputs(self, inputs):
        self._source.copyfile(self._target)



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import logging.config
import logging
import argparse
import sys 
import thread
import path
import traceback

import servicebot
import processbot
import configInstance
import busIndipendentMessages
import resource_cleaner

#-p d:\users\cimino\appdata\local\temp\wps_params_wprbze.tmp process
#-r .\\configs\\remote.config -s .\\configs\\RiskMaps\\service.config -p d:\\users\\cimino\\appdata\\local\\temp\\wps_params_q_245x.tmp process
#-r .\configs\remote.config -s .\configs\RiskMaps\service.config service
#-r .\configs\remote.config -s .\configs\OAAonDemand\service.config service
#-r .\configs\remote.config -s .\configs\OAAonDemand\service.config -p d:\users\cimino\appdata\local\temp\wps_params_ullgiw.tmp process

class WPSAgent(object):
    def __init__(self, args):
        #remove sleek xmpp logging 
        self.verbose = False
        self.args=args

    def find_logger_property_file(self, find_logger_property_file_path):
        if self.args.logconf == None:
            logger_properties_file = find_logger_property_file_path / "logger.properties"
        else:
            logger_properties_file = self.args.logconf
        return logger_properties_file
    
    @staticmethod
    def log_bootstrap_error( msg, stack_trace):
        sys.stderr.write(msg + "\n")
        sys.stderr.write(stack_trace + "\n")

    @staticmethod
    def set_resource_cleaner_parameters(pid_files_dir,process_time_threshold, workdir_time_threshold, sleep_time_seconds):
        resource_cleaner.Resource.pid_files_dir = pid_files_dir
        resource_cleaner.Resource.process_time_threshold = process_time_threshold
        resource_cleaner.Resource.workdir_time_threshold = workdir_time_threshold
        resource_cleaner.Resource.sleep_time_seconds = sleep_time_seconds

    @staticmethod
    def create_logger( logger_config_file, workdir, verbose):
        defaults={}
        if workdir != None:
            defaults['workdir'] = workdir
            #to avoid problems with escape sequence (\ab is apparently a valid hex number?!?!?), use always linux style path separator
            if '\\' in defaults['workdir']:
                defaults['workdir']=defaults['workdir'].replace('\\', '/')

        logging.config.fileConfig(str(logger_config_file),  defaults=defaults)

        logger = logging.getLogger("main.create_logger")
        if not verbose:
            #drop logs from sleekxmlpp
            for h in logger.root.handlers:
                h.addFilter(SleekXMPPLoggerFilter())

        logger.debug("Logger initialized with file " + str(logger_config_file))

    def create_bot(self):
        #template method
        pass

    def run(self):
        #we now have a logger
        logger = logging.getLogger("WPSAgent.run")
        try:
            bot = self.create_bot()
            logger.info("Start bot execution")
            bot.run()
        except Exception as e:
            msg = "Bot failure due to: " + str(e)
            logger.fatal(msg)
            logger.fatal( traceback.format_exc() )
            bot.send_error_message( msg )
            bot.disconnect()
            logger.fatal( "Exit Bot process with code " + str(101) )
            sys.exit(101)

class WPSAgentProcess(WPSAgent):
    """This script starts when the user call a new WPS execution. 
        His task is to call the proper external executable/scripts according to the service.config file (option -s in command line) and send back to the WPS logging and progress 
        information and error information if something unexpected happens.
        All the output including the log file is generated in a sand box directory created with joint information from service.config and external process start-up information.
        """
    def __init__(self, args):
        super(WPSAgentProcess, self).__init__(args)
        try:
            config_dir_service = path.path(args.serviceconfig).dirname()
            #deserilize pickled object with process startup info to get the unique_id to create the sand box work dir for the process execution
            self.exe_msg = busIndipendentMessages.ExecuteMessage.deserialize( args.params )
             
            #read the service config file with interpolation=true (raw=False) to get the proper sand box work dir using the unique id as input parameter
            #args.remoteconfig, args.serviceconfig
            serviceConfig = configInstance.create(args.serviceconfig , case_sensitive=True, variables = {'unique_exe_id' : self.exe_msg.UniqueId()}, raw=False) 
            work_dir = serviceConfig.get_path("DEFAULT", "workdir")

            #ensure outdir exists
            if not work_dir.exists():
                work_dir.mkdir()

            #now i can create a logger with output log file in the sandbox dir
            WPSAgent.create_logger( self.find_logger_property_file(config_dir_service), work_dir, self.verbose)

        except Exception as e:
            #here log is not available use log_bootstrap_error func to log somewhere else (e.g. operating system error log, tmp file, stdout)
            msg = "Failure during bot bootstrap due to : " + str(e)
            WPSAgent.log_bootstrap_error(msg, traceback.format_exc())
            sys.exit(100)
        
        #start execution
        self.run()

    def create_bot(self):
        try:
            logger = logging.getLogger("WPSAgentProcess.create_bot")
            logger.info("Create process bot")
            bot =  processbot.ProcessBot( self.args.remoteconfig, self.args.serviceconfig,  self.exe_msg )
            #create resource cleaner
            self.set_resource_cleaner_parameters(bot.get_resource_file_dir(), bot.max_execution_time(), bot.max_execution_time(), bot.max_execution_time())
            return bot
        except Exception as e:
            msg = "Failure during bot bootstrap due to : " + str(e)
            WPSAgent.log_bootstrap_error(msg, traceback.format_exc())
            sys.exit(100)

class WPSAgentService(WPSAgent):

    def __init__(self, args):
        try:
            super(WPSAgentService, self).__init__(args)
            remote_config_dir = path.path(args.remoteconfig).dirname()
            self.create_logger(self.find_logger_property_file( remote_config_dir ), None, self.verbose)
        except Exception as e:
            #here log is not available use log_bootstrap_error func to log somewhere else (e.g. operating system error log, tmp file, stdout)
            msg = "Failure during bot bootstrap due to : " + str(e)
            WPSAgent.log_bootstrap_error(msg, traceback.format_exc())
            sys.exit(100)

        #start execution
        self.run()

    def create_bot(self):
        try:
            logger = logging.getLogger("WPSAgentService.create_bot")
            logger.info("Create process bot")
            bot = servicebot.ServiceBot(self.args.remoteconfig, self.args.serviceconfig)
            logger.info("Create resource cleaner")
            WPSAgent.set_resource_cleaner_parameters(bot.get_resource_file_dir(), bot.max_execution_time(), bot.max_execution_time(), bot.max_execution_time())
            #start infinite loop for resource clenaer thread
            thread.start_new_thread( resource_cleaner.Resource.clean_up_all,())
            return bot
        except Exception as e:
            msg = "Failure during bot bootstrap due to : " + str(e)
            WPSAgent.log_bootstrap_error(msg, traceback.format_exc())
            sys.exit(100)

class SleekXMPPLoggerFilter(logging.Filter):

    def filter(self, record):
        return not 'sleekxmpp' in record.name 

if __name__ == '__main__':
    
    parser = argparse.ArgumentParser()
    parser.add_argument("-r", "--remoteconfig", nargs='?', default="remote.config", help="Config file containing connection information on the calling agent (e.g. XMPP server)")
    parser.add_argument("-s", "--serviceconfig",nargs='?', default="service.config", help="Config file containing information concerning the local process to be invoked by WPS")
    parser.add_argument("-l", "--logconf", nargs="?", default=None, help="Logger config file if not provided logger.conf in remoteconfig directory (runtype=service) or logger.conf in serviceconfig directory (runtype=process) will be used")
    parser.add_argument("-p", "--params", nargs="?", help="JSON file containing input parameters")
    parser.add_argument("-e", "--executionid", nargs="?", help="Unique execution id")
    parser.add_argument("runtype", help="Run type [service|process])") #, choices=['service', 'process'])
    cmdargs = parser.parse_args()

    if cmdargs.runtype=='service':
        WPSAgentService(cmdargs)
    else: #process
        WPSAgentProcess(cmdargs)



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

class Bus(object):
    def __init__(self, id):
        self.callbacks={}
        self.id = id

    def Listen(self):
        pass

    def CheckServerIdentity(self, serverId):
        pass

    def SendMessage(self,Message):
        pass

    def Stop(self, Message):
        pass

    def RegisterMessageCallback(self, message_type, callback):
        self.callbacks[message_type] = callback



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import sys,os
import logging, ConfigParser, thread, subprocess, inspect, pickle, json, socket, zipfile, shutil, urllib, re, time, hashlib, urllib, base64

os.chdir(os.path.join(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))))
sys.path.insert(0, os.getcwd())
sys.path.append(os.path.abspath('lib'))

import sleekxmpp
import logging
import logging.config

class ServiceTester(sleekxmpp.ClientXMPP):

    def __init__(self, jid, password):
        sleekxmpp.ClientXMPP.__init__(self, jid, password)
        self.jid=jid
        self.register_plugin('xep_0004') # Data forms
        self.register_plugin('xep_0030') # Service Discovery
        self.register_plugin('xep_0045') # MUC
        self.register_plugin('xep_0060') # MUC
        self.register_plugin('xep_0077') # In-band Registration
        self.register_plugin('xep_0078') # Non-SASL Authentication
        self.register_plugin('xep_0199') # XMPP Ping
        self.register_plugin('xep_0249') # Direct MUC invitation
        self.add_event_handler("session_start", self.start)
        self.add_event_handler("message", self.message)

    def start(self, event):
        self.send_presence()
        self.get_roster()

    def message(self, signal):
      log=logging.getLogger("serviceTester")
      if signal['type'] in ('normal', 'chat'):
                line = signal['body'] 
                log.info('Received XMPP bus signal: "%s"', line)
                self.handleSignal(line, signal['from'])
                 
    def run(self):
        if self.connect(("172.21.160.51",5223),use_tls=False, use_ssl=True):
            self.channelJid = "default@conference.whale.nurc.nato.int"
            self.busResource = "admin@whale.nurc.nato.int"
            self.plugin['xep_0045'].joinMUC(self.channelJid, self.busResource, password="Crociera100!")

            self.add_event_handler('muc::%s::got_online' % self.channelJid, self.gotOnline)

            print("in the loop...")
            self.process(block=True)
        else:
            print("Unable to connect.")

    def gotOnline(self, presence):
        ''' If got online event fired:
                - if self gotOnline: configureChat
                - if service got online: invite
        
            @param msg: the got online msg
        '''
        if 'master' in str(presence['muc']['nick']):
            # Send XMPP invitation
            self.sendInvitation(presence['from'])

    def sendInvitation(self, servicejid):
        ''' send invitation to a service to register with the orchestrator
        
            @param jid: the jid which should be invited
        '''
        log=logging.getLogger("serviceTester")
        log.debug('Send Invitation to %s', servicejid)
        self.send_message(servicejid, mbody='topic=invite')
        pass

    def handleSignal(self, signal, origin):
        ''' Handle an incoming signal either via IPC BUS or XMPP
            @param signal: the message body
        '''
        
        signalArgs = None
        try:
            signalArgs = dict([tuple(each.strip().split('=')) for each in signal.split('&')])
        except:
            logging.error('Bus signal "%s" not supported (Error: %s)', signal, sys.exc_info()[0])
        
        # This is a list of supported bus signals that trigger an action  
        if signalArgs is not None:
            # Handle topic
            if signalArgs.has_key('topic'):
                if signalArgs['topic'] == 'register':
                    logging.info('Registering remote service "%s"' % signalArgs['service'])
                    logging.debug('Received a request for "%s"', signalArgs['service'])
                    self.invokeService(signalArgs['service'], origin)

                elif signalArgs['topic'] == 'progress':
                    logging.debug('received progress msg with value %s', signalArgs['message'])

                elif signalArgs['topic'] == 'log':
                    logging.debug('Received log msg with level %s msg=%s', signalArgs['level'],signalArgs['message'] )

                elif signalArgs['topic'] == 'completed':
                    msgbody='topic=finished' + '&id=' + self.pid
                    logging.debug( msgbody )
                    self.send_message(origin, mbody=msgbody)
                elif signalArgs['topic'] == 'error':
                    logging.debug( "**** error ****" )
                    logging.debug( msgbody )

                        
              
            else:
                logging.warn('Bus signal "%s" provides no topic', signal)

    def invokeService(self, service, origin):

        request = self.request_oaa_custom_riskmap()

        requestPickle = pickle.dumps(request)
        msgPickledURLEnc=urllib.quote(requestPickle)

        self.pid = hashlib.md5(str(time.time())).hexdigest()[-5:] + '_' + hashlib.md5('%s%s' % (service, urllib.quote(msgPickledURLEnc))).hexdigest()

        # Send Request to the Service
        msgbody='topic=request&service=' + service + '&id=' + self.pid + '&message=' + msgPickledURLEnc + '&baseURL=localhost'
        print msgbody
        self.send_message(origin, mbody=msgbody)

    def request_ok(self):
        request = {u'asset': [u'{\n    "Asset": {\n        "Pfa": 0.001, \n        "maxSpeed": 10, \n        "minSpeed": 2, \n        "maxHeading": 2.1, \n        "minHeading": -2.1, \n        "lat0": 10.0, \n        "cost": 1e-06, \n        "obsRange": 50000, \n        "heading0": 0, \n        "lon0": 62.0, \n        "type": "Frigate", \n        "id": 1, \n        "Pd": 0.6, \n        "name": "Ship 1"\n    }\n}',
            u'{\n    "Asset": {\n        "Pfa": 0.001, \n        "maxSpeed": 10, \n        "minSpeed": 2, \n        "maxHeading": 2.1, \n        "minHeading": -2.1, \n        "lat0": 10.0, \n        "cost": 1e-06, \n        "obsRange": 50000, \n        "heading0": 0, \n        "lon0": 60.0, \n        "type": "Frigate", \n        "id": 2, \n        "Pd": 0.6, \n        "name": "Ship 2"\n    }\n}',
            u'{\n    "Asset": {\n        "Pfa": 0.001, \n        "maxSpeed": 10, \n        "minSpeed": 2, \n        "maxHeading": 2.1, \n        "minHeading": -2.1, \n        "lat0": 5.0, \n        "cost": 1e-06, \n        "obsRange": 50000, \n        "heading0": 0, \n        "lon0": 58.0, \n        "type": "Frigate", \n        "id": 3, \n        "Pd": 0.6, \n        "name": "Ship 3"\n    }\n}',
            u'{\n    "Asset": {\n        "Pfa": 0.001, \n        "maxSpeed": 10, \n        "minSpeed": 2, \n        "maxHeading": 2.1, \n        "minHeading": -2.1, \n        "lat0": 5.0, \n        "cost": 1e-06, \n        "obsRange": 50000, \n        "heading0": 0, \n        "lon0": 58.0, \n        "type": "Frigate", \n        "id": 4, \n        "Pd": 0.6, \n        "name": "Ship 4"\n    }\n}'],
            'numberOfEvaluations': 600,
            'referenceDate': '20140901',
            'timeHorizon': 259200,
            'minlon' : 50.0,
            'maxlon' : 70.0,
            'minlat' : 0,
            'maxlat' : 20.0
            }
        return request

    def request_error(self):
        request = {u'asset': [
            u'{\n    "Asset": {\n        "Pfa": 1, \n        "maxSpeed": 1, \n        "minSpeed": 1, \n        "maxHeading": 1, \n        "minHeading": 1, \n        "lat0": 10.0, \n       "cost": 1, \n        "obsRange": 1, \n        "heading0": 1, \n        "lon0": 62.0, \n        "type": "Frigate", \n        "id": 1, \n        "Pd": 1, \n        "name": "Ship 1"\n    }\n}',
            u'{\n    "Asset": {\n        "Pfa": 1, \n        "maxSpeed": 1, \n        "minSpeed": 1, \n        "maxHeading": 1, \n        "minHeading": 1, \n        "lat0": 10.0, \n       "cost": 1, \n        "obsRange": 1, \n        "heading0": 1, \n        "lon0": 60.0, \n        "type": "Frigate", \n        "id": 2, \n        "Pd": 1, \n        "name": "Ship 2"\n    }\n}',
            u'{\n    "Asset": {\n        "Pfa": 1, \n        "maxSpeed": 1, \n        "minSpeed": 1, \n        "maxHeading": 1, \n        "minHeading": 1, \n        "lat0": 5.0, \n        "cost": 1, \n        "obsRange": 1, \n        "heading0": 1, \n        "lon0": 58.0, \n        "type": "Frigate", \n        "id": 3, \n        "Pd": 1, \n        "name": "Ship 3"\n    }\n}',
            u'{\n    "Asset": {\n        "Pfa": 1, \n        "maxSpeed": 1, \n        "minSpeed": 1, \n        "maxHeading": 1, \n        "minHeading": 1, \n        "lat0": 5.0, \n        "cost": 1, \n        "obsRange": 1, \n        "heading0": 1, \n        "lon0": 58.0, \n        "type": "Frigate", \n        "id": 4, \n        "Pd": 1, \n        "name": "Ship 4"\n    }\n}'],
            'numberOfEvaluations': 600,
            'referenceDate': '20140901',
            'timeHorizon': 259200,
            'minlon' : 50.0,
            'maxlon' : 70.0,
            'minlat' : 0,
            'maxlat' : 20.0
            }
        return request

    def request_riskmap(self):
        request = {
            'referenceDate' : '20150115',
            'minlon' : 50.0,
            'maxlon' : 70.0,
            'minlat' : 0,
            'maxlat' : 20.0,
            'baseWCSURL' : 'karapthos',
            'layerName' : 'testlayer',
            'outputFormat' : 'GeoTiff',
            'requestCRS' : '3426',
            'outputCRS' : '4326',
            'resX' :  200,
            'resY' :  200,
            'workdir' : 'd:\\tmp2\\'
            }

    def request_oaa_custom_riskmap(self):
        request = {
            'referenceDate' : '20150115',
            'numberOfEvaluations': 600,
            'timeHorizon': 259200,
            u'asset': [
                u'{\n    "Asset": {\n        "Pfa": 1, \n        "maxSpeed": 1, \n        "minSpeed": 1, \n        "maxHeading": 1, \n        "minHeading": 1, \n        "lat0": 10.0, \n       "cost": 1, \n        "obsRange": 1, \n        "heading0": 1, \n        "lon0": 62.0, \n        "type": "Frigate", \n        "id": 1, \n        "Pd": 1, \n        "name": "Ship 1"\n    }\n}',
                u'{\n    "Asset": {\n        "Pfa": 1, \n        "maxSpeed": 1, \n        "minSpeed": 1, \n        "maxHeading": 1, \n        "minHeading": 1, \n        "lat0": 10.0, \n       "cost": 1, \n        "obsRange": 1, \n        "heading0": 1, \n        "lon0": 60.0, \n        "type": "Frigate", \n        "id": 2, \n        "Pd": 1, \n        "name": "Ship 2"\n    }\n}',
                u'{\n    "Asset": {\n        "Pfa": 1, \n        "maxSpeed": 1, \n        "minSpeed": 1, \n        "maxHeading": 1, \n        "minHeading": 1, \n        "lat0": 5.0, \n        "cost": 1, \n        "obsRange": 1, \n        "heading0": 1, \n        "lon0": 58.0, \n        "type": "Frigate", \n        "id": 3, \n        "Pd": 1, \n        "name": "Ship 3"\n    }\n}',
                u'{\n    "Asset": {\n        "Pfa": 1, \n        "maxSpeed": 1, \n        "minSpeed": 1, \n        "maxHeading": 1, \n        "minHeading": 1, \n        "lat0": 5.0, \n        "cost": 1, \n        "obsRange": 1, \n        "heading0": 1, \n        "lon0": 58.0, \n        "type": "Frigate", \n        "id": 4, \n        "Pd": 1, \n        "name": "Ship 4"\n    }\n}'
            ],
            'minlon' : 50.0,
            'maxlon' : 70.0,
            'minlat' : 0,
            'maxlat' : 20.0,
            'owsBaseURL' : 'http://karpathos-dev/geoserver/wcs',
            'owsService' : 'WCS',
            'owsVersion' : '1.0.0',
            'requestCRS' : 'EPSG:4326',
            'timeResolution' : 3,
            'owsResourceIdentifier' : 'TDA:PAGM'
        }

        return request

if __name__ == '__main__':
    logging.config.fileConfig("logger_serviceTester.properties")
    log=logging.getLogger("serviceTester")
    log.debug("start tester")
    
    xmpp = ServiceTester("admin@whale.nurc.nato.int","Crociera100!")
    
    xmpp.run()



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

from collections import OrderedDict
import path
import introspection
import ConfigParser

def action_factory(config_section_items):
    actions = OrderedDict()
    for a in config_section_items:
        d = dict(config_section_items[a])
        if ('type' in d):
            actions[a] = introspection.get_class_one_arg( d['type'], d )

    return actions

class CopyFile(object):
    def __init__(self, param_dict):
        self.source = path.path(param_dict['source'])
        self.target = path.path(param_dict['target'])

    def execute(self, input_values):
         self.source.copy(self.target)


class CopyINIFileAddParam(object):
    def __init__(self, param_dict):
        self.source = path.path(param_dict['source'])
        self.target = path.path(param_dict['target'])
        self.param_section = param_dict['param_section']
        self.param_name = param_dict['param_name']
        self.param_value_ref = param_dict['param_value_ref']

    def execute(self, input_values):
        self.source.copy(self.target)
        config = ConfigParser.ConfigParser(allow_no_value=True)
        fp = self.target.open() 
        config.readfp( fp  )
        fp.close()

        config.set(self.param_section, self.param_name, input_values[self.param_value_ref])
       
        with self.target.open('wb') as configfile:
            config.write(configfile)


# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import threading
import thread
import time
import psutil
import logging

logger = logging.getLogger("servicebot.resource_monitor")


class ResourceMonitor(threading.Thread):

    load_average_scan_minutes = 15
    cores = psutil.cpu_count()
    cpu_perc  = []
    vmem_perc = []
    lock = threading.Lock()

    def __init__(self, load_average_scan_minutes):
        threading.Thread.__init__(self)
        ResourceMonitor.load_average_scan_minutes = load_average_scan_minutes
        ResourceMonitor.lock.acquire()

        ResourceMonitor.vmem_perc.append(psutil.virtual_memory().percent)
        ResourceMonitor.vmem_perc.append(psutil.virtual_memory().percent)

        ResourceMonitor.cpu_perc.append(psutil.cpu_percent(interval = 0, percpu= False))
        ResourceMonitor.cpu_perc.append(psutil.cpu_percent(interval = 0, percpu= False))

        ResourceMonitor.lock.release()

    def proc_is_running(self, proc_defs):
        for proc in psutil.process_iter():
            try:
                process = psutil.Process(proc.pid) # Get the process info using PID
                if process.is_running():
                    pid    = str(process.pid)
                    ppid   = str(process.ppid)
                    status = process.status()

                    cpu_percent = process.cpu_percent()
                    mem_percent = process.memory_percent()

                    rss = str(process.memory_info().rss)
                    vms = str(process.memory_info().vms)
                    username = process.username()
                    name     = process.name() # Here is the process name
                    path     = process.cwd()
                    cmdline  = ' '.join(process.cmdline())

                    print("Get the process info using (path, name, cmdline): [%s / %s / %s]" % (path, name, cmdline))
                    for _p in proc_defs:
                        # logger.info("Look for process: [%s] / Status [%s]" % (_p, status.lower()))
                        # print("Look for process: [%s] / Status [%s]" % (_p, status.lower()))
                        if (status.lower() != "sleeping") and \
                            ('name' in _p and _p['name'] in name) and \
                                ('cwd' in _p and _p['cwd'] in path) and \
                                    ('cmdline' in _p and _p['cmdline'] in cmdline):
                            return True
            except:
                import traceback
                tb = traceback.format_exc()
                logger.debug(tb)
                print(tb)
        return False

    def run(self):
        while True:
            ResourceMonitor.lock.acquire()

            ResourceMonitor.vmem_perc[1] = (ResourceMonitor.vmem_perc[0] + ResourceMonitor.vmem_perc[1]) / 2.0
            ResourceMonitor.vmem_perc[0] = (ResourceMonitor.vmem_perc[1] + psutil.virtual_memory().percent) / 2.0

            ResourceMonitor.cpu_perc[1] = ResourceMonitor.cpu_perc[0]
            ResourceMonitor.cpu_perc[0] = psutil.cpu_percent(interval = (ResourceMonitor.load_average_scan_minutes*60), percpu= False)

            ResourceMonitor.lock.release()



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

"""todo: move this module out of  the project in cmre dedicated section"""
import json
import unittest
import datetime

import path

import config_file_parameter
import input_parameters

class CMREOAA_AssetConfigFile(config_file_parameter.ConfigFileParameter):

    shrot_name = "OAAAssetFile"

    def __init__(self, name):
        super(CMREOAA_AssetConfigFile, self).__init__(name)
        self._template_file_path = None

    def _get_file_path(self, id):
        return path.path(self._filepath.replace(self._template_file_path, str(id)))

    def update_file(self):
        asset_id=0
        for v in self._value:
            asset_json=json.loads(v)
            asset_id = int(asset_json['Asset']['id'])
            asset_file= self._get_file_path(asset_id)
            #asset_file.write_text( v )
            with asset_file.open('wb') as outfile:
                json.dump(asset_json, outfile, indent=4)



class CMREOAA_MainConfigFile(config_file_parameter.ConfigFileParameter):

    shrot_name = "OAAConfigFile"

    def __init__(self, name):
        super(CMREOAA_MainConfigFile, self).__init__(name)
        self._config_file_template = path.path('CMREOAA_MainConfigFile_template.json')

    def inject_values(self, paremeters_types_defs):
        super(CMREOAA_MainConfigFile, self).inject_values(paremeters_types_defs)
        if (self._filepath.exists()):
            self._filepath.remove()
      

    def update_file(self):
        #if target files exists open it and update, if doesnt exists load the template, modify it and save to target (it is useful for multiple update on the same file)        
        if (self._filepath.exists()):
            template = json.load(self._filepath.open())
        else:
            template = json.load(self._config_file_template.open())


        if (self.get_name() == 'timeHorizon'):
            template['Config']['timeHorizon'] = int(self._value[0])

        elif (self.get_name() == 'nEvaluations'):
            template['Config']['nEvaluations'] = int(self._value[0])
        else:
            pass #todo: log "Cannot write input pararmeter " + name + " in config file " + str( self._config_file_path )

        with self._filepath.open('wb') as outfile:
            json.dump(template, outfile, indent=4)




class Test(unittest.TestCase):

    def test_asset_config_object(self):

        a1=path.path("d:\\tmp\\asset_1.json")
        a2=path.path("d:\\tmp\\asset_2.json")

        if a1.exists():
            a1.remove()
        if a2.exists():
            a2.remove()

        paramsDef = [('assets' , {"type": "application/json",  "description": "Assets config", "max": 10, "filepath" : "d:\\tmp\\asset_$i.json", "template_file_path" : "$i", "class" : "config_file_parameter_cmre_oaa.CMREOAA_AssetConfigFile"} ) ]

        paramValue = {'assets': ['''{
            "Asset": {
                "Pfa": 0.001, 
                "maxSpeed": 10, 
                "minSpeed": 2, 
                "maxHeading": 2.1, 
                "minHeading": -2.1, 
                "lat0": 12.08, 
                "cost": 1e-06, 
                "obsRange": 50000, 
                "heading0": 0, 
                "lon0": 63.17, 
                "type": "Frigate", 
                "id": 1, 
                "Pd": 0.6, 
                "name": "Ship 1"
            }
        }''', '''{
            "Asset": {
                "Pfa": 0.001, 
                "maxSpeed": 10, 
                "minSpeed": 2, 
                "maxHeading": 2.1, 
                "minHeading": -2.1, 
                "lat0": 13.66, 
                "cost": 1e-06, 
                "obsRange": 50000, 
                "heading0": 0, 
                "lon0": 58.9, 
                "type": "Frigate", 
                "id": 2, 
                "Pd": 0.6, 
                "name": "Ship 2"
            }
        }'''], 
        'numberOfEvaluations': 600, 'timeHorizont': 259200}

        pip = input_parameters.InputParameters(paramsDef)
        pip.parse( paramValue )
        
        res = pip.get_cmd_line()

        self.assertEquals(res.strip(), "")

        self.assertTrue(a1.exists())
        self.assertTrue(a2.exists())

    def test_OAA_main_config_file(self):
        a1=path.path("d:\\tmp\\oaa_config.json")

        if a1.exists():
            a1.remove()


        paramsDef = [
            ('numberOfEvaluations',   {"type": "int", "description": "number of optimizer iterations", "max": 1,  "alias" : "nEvaluations", "class" : "config_file_parameter_cmre_oaa.CMREOAA_MainConfigFile", "filepath" : "d:\\tmp\\oaa_config.json" }), 
            ('referenceDate',         {"type": "datetime", "description": "start date of optimizer", "max": 1, "alias" : "refDate", "formatter" : "%Y%m%d", "class" : "config_file_parameter_cmre_oaa.CMREOAA_MainConfigFile", "filepath" : "d:\\tmp\\oaa_config.json" }), 
            ('timeHorizon',           {"type": "int", "description": "Time horizon in seconds of the optimizer", "max": 1, "class" : "config_file_parameter_cmre_oaa.CMREOAA_MainConfigFile", "filepath" : "d:\\tmp\\oaa_config.json" }),
            ('executiontType',        {"type": "string", "description": "etl execution type", "max": 1, "template" : "value"})
        ]

        paramValue = {'argx' : 'test' }
        paramValue = { 'numberOfEvaluations': 600, 'referenceDate' : '20140901', 'timeHorizon': 259200, 'executiontType' : 'oaaOnDemand' }

        pip = input_parameters.InputParameters(paramsDef)
        pip.parse( paramValue )
        
        res = pip.get_cmd_line()


        self.assertEquals(res.strip(), "--referencedate==20140901 oaaOnDemand" )

        self.assertTrue( a1.exists() )

        configs  = json.load( a1.open() )
        self.assertEqual(configs['Config']['nEvaluations'], 600)
        self.assertEqual(configs['Config']['refDate'], '2014-09-01T00:00:00Z')
        self.assertEqual(configs['Config']['timeHorizon'], 259200)


if __name__ == '__main__':
    unittest.main()


# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import unittest
import output_file_parameter
import path
from collections import OrderedDict
import logging

class OutputParameters(object):

    @staticmethod
    def create_from_config(output_sections, wps_execution_shared_dir=None, uploader=None):
        '''Static constructor.
        input_sections: a dictionary such as { par1_name : [( 'par1_attrubute1_name' : par1_attrubute1_value ), ( 'par2_attrubute1_name' : par2_attrubute1_value ), ...], par2_name : { ... }, ... }
        '''
        output_sections_reshaped={}
        #force the order of sections in config files: [output1], [output2], etc
        for k in sorted(output_sections):
            d=dict(output_sections[k])
            name = d['name']
            del d['name']
            output_sections_reshaped[name] = d
        return OutputParameters( output_sections_reshaped, wps_execution_shared_dir, uploader )

    def __init__(self, parameters_types_defs, wps_execution_shared_dir, uploader):
        '''
        Static constructor.
        parameters_types: ('out_par_name1', {"out_par_name1_attrib1": "value1", "out_par_name1_attrib1": "value2", ...})
        '''
        self._wps_execution_shared_dir=wps_execution_shared_dir;
        self._params={}

        for name, d in OrderedDict(parameters_types_defs).items():
            if ('type' in d):

                if ('string' == d['type']):
                    self._params[name] = output_file_parameter.OutputFileParameter( name, d, wps_execution_shared_dir=self._wps_execution_shared_dir, uploader=uploader)

                elif ('image/geotiff' in d['type'] or 'text/xml' in d['type'] or 'application/gml' in d['type'] or 'application/zip' in d['type'] or 'application/x-netcdf' in d['type'] or 'video/mp4' in d['type']):
                    self._params[name] = output_file_parameter.RawFileParameter( name, d, wps_execution_shared_dir=self._wps_execution_shared_dir, uploader=uploader)

                elif ('application/owc' in d['type']):
                    self._params[name] = output_file_parameter.OWCFileParameter( name, d, parameters_types_defs, wps_execution_shared_dir=self._wps_execution_shared_dir, uploader=uploader)

                else:
                    logger = logging.getLogger("OutputParameters.__init__")
                    msg="Unknown output parameter " + name + " type"
                    logger.critical(msg)
                    raise Exception(msg)
            else:
                pass

    def get_values(self):
        res=[]
        for k in self._params:
            res.append(self._params[k].get_value())
        return res

    def parameters(self):
        for p in self._params.values():
            yield p

    def as_DLR_protocol(self):
        #[('result', '{"type": "string", ...ut.xml" }')]
        res = []
        for k in self._params.keys():
            n = self._params[k].get_name()
            j = self._params[k].as_json_string()
            res.append( (n,j) )
        return res

class Test(unittest.TestCase):
    
    def test_output_par_from_file(self):

        f=path.path("d:\\tmp\\test.txt")
        msg="this is a test"
        f.write_text(msg)
        
        p = OrderedDict( {"type": "string", "description": "xml OAA output file", "filepath" : "d:\\tmp\\test.txt"}   )
        par_types = [('result', p)]

        pop = OutputParameters(par_types)

        it = pop.parameters()
        val = next(it)
        self.assertEqual( val, msg)

        f.remove()

if __name__ == '__main__':
    unittest.main()



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import pickle

class BusInipendentMessage(object):
    pass

class PresenceMessage(BusInipendentMessage):
    pass

class InviteMessage(BusInipendentMessage):

    def __init__(self, payload, originator):
        self._originator=originator
        self._payload=payload

    def originator(self):
        return self._originator

class RegisterMessage(BusInipendentMessage):

    def __init__(self, originator, service, namespace, descritpion, par, output):
        self._originator = originator
        self.service = service
        self.namespace = namespace
        self.description = descritpion
        self._input_parameter = par
        self.output = output

    def input_parameters(self):
        return self._input_parameter

    def originator(self):
        return self._originator

class ExecuteMessage(BusInipendentMessage):

    @staticmethod
    def deserialize(filepath):
        fp=open(filepath)
        #exe_msg = json.load(json_file)
        exe_msg = pickle.load(fp)
        fp.close()
        return exe_msg

    def __init__(self, originator, uniqueExeId, baseURL, variables):
        self._uniqueExeId = uniqueExeId
        self._baseURL = baseURL
        self._originator = originator
        self._variables = variables

    def variables(self):
        return self._variables

    def originator(self):
        return self._originator

    def UniqueId(self):
        return self._uniqueExeId

    def BaseURL(self):
        return self._baseURL

    def serialize(self, fileptr):
        #json.dump( self._originator , fileptr)
        pickle.dump(self, fileptr)

class ProgressMessage(BusInipendentMessage):

   def __init__(self,  originator, progress):
        self.originator = originator
        self.progress = progress

class LogMessage(BusInipendentMessage):

   def __init__(self,  originator, level, msg):
        self.level = level
        self.originator = originator
        self.msg = msg

class CompletedMessage(BusInipendentMessage):

    def __init__(self, originator, base_url, outputs):
        self.originator = originator
        self.base_url=base_url
        self._outputs = outputs

    def outputs(self):
        return self._outputs

class FinishMessage(BusInipendentMessage):

    def __init__(self, payload, originator):
        self.originator=originator
        self.payload=payload

class ErrorMessage(BusInipendentMessage):

    def __init__(self, originator, msg, id=None):
        self.originator = originator
        self.msg = msg
        self.id = id

class AbortMessage(BusInipendentMessage):

    def __init__(self, payload, originator):
        self.originator=originator
        self.payload=payload

class GetLoadAverageMessage(BusInipendentMessage):

    def __init__(self, payload, originator):
        self._originator=originator
        self._payload=payload

    def originator(self):
        return self._originator

class LoadAverageMessage(BusInipendentMessage):

    def __init__(self, originator, outputs):
        self.originator = originator
        self._outputs = outputs

    def outputs(self):
        return self._outputs



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

from collections import OrderedDict
import path

import computational_job_input_action_cmd_param
import computational_job_input_action_create_json_file
import computational_job_input_action_update_json_file
import computational_job_input_action_copyfile
import computational_job_input_action_update_ini_file

class ComputationalJobInputActions(object):

    @staticmethod
    def create_from_config(input_sections):
        '''Create a InputParameters object.

        input_sections: a dictionary such as { action1 : [( 'par1_input1_name' , par1_input1_value ), ( 'par2_input1_name' , par2_input1_value ), ...], action2 : [ .... ], ... }
        '''
        input_sections_reshaped=OrderedDict()
        #force the order of sections in config files: [action1], [action2], etc
        for k in sorted(input_sections):
            d=dict(input_sections[k])
            #input_ref = d['input_ref']
            #del d['input_ref']
            input_sections_reshaped[k] = d
        return ComputationalJobInputActions.create_from_dict( input_sections_reshaped )

    @staticmethod
    def create_from_dict(input_action_def_dict):
        '''Create a input parameters set from a definitions

        paremeters_types_defs is a dictionary such as {input_ref1 : { 'action1_attrubute1_name' : action1_attrubute1_value, ... }, input_ref2 : { 'action2_attrubute1_name' : action2_attrubute1_value, ... }, ... }
        '''
        cja = ComputationalJobInputActions()
        action2add = None
        for action_section_name, d in OrderedDict(input_action_def_dict).items():
            if ('class' in d):
                if d['class'] == 'cmdline':
                    alias = d['alias'] if "alias" in d else None
                    action2add = computational_job_input_action_cmd_param.ComputationalJobInputActionCmdParam( d['input_ref'], d['template'], alias = alias ) 
                elif d['class'] == 'createJSONfile':
                    #remove_if_exists = d['remove_if_exists'] if "remove_if_exists" in d else None
                    json_path_expr = d['json_path_expr'] if "json_path_expr" in d else None
                    action2add = computational_job_input_action_create_json_file.ComputationalJobInputActionCreateJSONFile(d['input_ref'], d['target_filepath'], json_path_expr, path.path(d['json_schema']) )
                elif d['class'] == 'updateJSONfile':
                    action2add = computational_job_input_action_update_json_file.ComputationalJobInputActionUpdateJSONFile(  d['input_ref'], d['target_filepath'], d['json_path_expr'], d['source_filepath'] )
                elif d['class'] == 'copyfile':
                    action2add = computational_job_input_action_copyfile.ComputationalJobInputActionCopyFile( d['source_filepath'], d['target_filepath'] )
                elif d['class'] == 'updateINIfile':
                    alias = d['alias'] if "alias" in d else None
                    action2add = computational_job_input_action_update_ini_file.ComputationalJobInputActionUpdateINIFile( d['input_ref'], d['source_filepath'], d['target_filepath'], d['section'], alias)
                elif d['class'] == 'updateINIfileList':
                    alias = d['alias'] if "alias" in d else None
                    action2add = computational_job_input_action_update_ini_file.ComputationalJobInputActionUpdateINIFileAsList( d['input_ref'], d['source_filepath'], d['target_filepath'], d['section'], alias)
                else:
                    raise TypeError("Unknown class value "+ str(d['class']) + " for action related to input " + str(d['input_ref']))
            else:
                raise TypeError("Cannot create computational job action related to input " + str(d['input_ref']) + " without attribute class")
            cja.add_actions( action2add )
        return cja

    def __init__(self):
        self._actions = []

    def get_cmd_line(self):
        cmd_line = ''
        for a in self._actions:
            if a.can_produce_cmd_line():
                cmd_line += ' ' + a.get_cmd_line()

        return cmd_line.lstrip()

    def execute(self, input_values):
        for a in self._actions:
            a.set_inputs( input_values )

    def add_actions(self, a):
        self._actions.append(a)



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import SocketServer
import socket
import threading
import path 
import os
import thread
import datetime

class  TerminationHandler(object):
    def __init__(self, handler):
        self._handler = handler

        HOST, PORT = "localhost", 0

        setattr(ThreadedTCPRequestHandler, '_handle', handler)

        server = ThreadedTCPServer((HOST, PORT), ThreadedTCPRequestHandler)
        self.ip, self.port = server.server_address
        self._write_pid_file()
        # Start a thread with the server -- that thread will then start one
        # more thread for each request
        server_thread = threading.Thread(target=server.serve_forever)
        # Exit the server thread when the main thread terminates
        server_thread.daemon = True
        server_thread.start()

    def _write_pid_file(self):
        pid = os.getpid()
        workdir = path.path("d:\\tmp\\wpsproc\\")
        pidfile = workdir / path.path(str(pid) +  ".pid")

        if pidfile.exists():
            pidfile.remove()

        pidfile.write_lines( ['cmdline=', 'pid='+str(pid), 'start_time=' + str(datetime.datetime.now()), 'tcp_port=' + str(self.port) ] )


class ThreadedTCPRequestHandler(SocketServer.BaseRequestHandler):
    
    def handle(self):
        data = self.request.recv(1024)

        #if (data=="kill"):
        cur_thread = threading.current_thread()
        response = "{}: {}".format( cur_thread.name, self._handle() )
        self.request.sendall(response)

class ThreadedTCPServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
    pass

def clean_up1(threadedTCPRequestHandler):
    return "clean_up1"

def clean_up2(threadedTCPRequestHandler):
    return "clean_up2"


def client(ip, port, message):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect((ip, port))

    try:
        sock.sendall(message)
        response = sock.recv(1024)
        print "Received: {}".format(response)
    finally:
        sock.close()

# #############################################################

class WatchDog(object):
    """
        run as service every 15 minutes

        #iterate over pids of all process bots
        for pid in path.path("d:\\tmp\\wpsproc\\").files(): #[1.pid, 2.pid,] 
            read file:
                cmdline=...
                pid=...
                start_time=...
                tcp port=...

            if pid is running and cmdline is ok and time is expired:
                #kill
                open socket(port)
                write "kill" to socket

                
            elif pids is not running:
                del "/var/proc/wpsproc/<pid>.pid

           
    """
    pass

# #############################################################

if __name__ == '__main__':
    # Port 0 means to select an arbitrary unused port

    th =TerminationHandler(clean_up1)


    client(th.ip, th.port, "Hello World 1")
    client(th.ip, th.port, "Hello World 2")
    client(th.ip, th.port, "Hello World 3")

    #server.shutdown()

    #th.set_handler(clean_up2)


# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Ned Batchelder"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

"""
FtpsUpload

Upload files via FTPS based on their content changing.

Based on original code by
   Ned Batchelder
   http://www.nedbatchelder.com
   version = '1.0a'

Modified by
   Alessio Fabiani, GeoSolutions S.A.S.
"""

import ftpUpload
import ftplib, socket, ssl, pickle, sys, md5, os, string
import logging      # if not std, http://www.red-dove.com/python_logging.html
import path         # http://www.jorendorff.com/articles/python/path
import upload


class tyFTP(ftplib.FTP_TLS):
    def __init__(self,
                 host='',
                 user='',
                 passwd='',
                 acct='',
                 keyfile=None,
                 certfile=None,
                 timeout=60):

        ftplib.FTP_TLS.__init__(self,
                                host=host,
                                user=user,
                                passwd=passwd,
                                acct=acct,
                                keyfile=keyfile,
                                certfile=certfile,
                                timeout=timeout)

    """
     - This is for Implicit TLS
    def connect(self, host='', port=0, timeout=-999):
        ""Connect to host.  Arguments are:
         - host: hostname to connect to (string, default previous host)
         - port: port to connect to (integer, default previous port)
        ""
        if host != '':
            self.host = host
        if port > 0:
            self.port = port
        if timeout != -999:
            self.timeout = timeout
        try:
            self.sock = socket.create_connection((self.host, self.port), self.timeout)
            self.af = self.sock.family
            # add this line!!!
            self.sock = ssl.wrap_socket(self.sock,
                                        self.keyfile,
                                        self.certfile,
                                        ssl_version=ssl.PROTOCOL_TLSv1)
            # add end
            self.file = self.sock.makefile('rb')
            self.welcome = self.getresp()
        except:
            logging.exception('Got exception on "tyFTP.connect"')
            logging.info(self.ssl_version)
            raise

        return self.welcome
    """

    def storbinary(self, cmd, fp, blocksize=8192, callback=None, rest=None):
        self.voidcmd('TYPE I')
        conn = self.transfercmd(cmd, rest)
        try:
            while 1:
                buf = fp.read(blocksize)
                if not buf: break
                conn.sendall(buf)
                if callback: callback(buf)
            # shutdown ssl layer
            if isinstance(conn, ssl.SSLSocket):
                conn.unwrap()
        finally:
            conn.close()
        return self.voidresp()

    def storlines(self, cmd, fp, callback=None):
        self.voidcmd('TYPE A')
        conn = self.transfercmd(cmd)
        try:
            while 1:
                buf = fp.readline(self.maxline + 1)
                if len(buf) > self.maxline:
                    raise Error("got more than %d bytes" % self.maxline)
                if not buf: break
                if buf[-2:] != CRLF:
                    if buf[-1] in CRLF: buf = buf[:-1]
                    buf = buf + CRLF
                conn.sendall(buf)
                if callback: callback(buf)
            # shutdown ssl layer
            if isinstance(conn, ssl.SSLSocket):
                conn.unwrap()
        finally:
            conn.close()
        return self.voidresp()    


class FtpsUpload(ftpUpload.FtpUpload):
    """
    Provides intelligent FTPS uploading of files, using MD5 hashes to track
    which files have to be uploaded.  Each upload is recorded in a local
    file so that the next upload can skip the file if its contents haven't
    changed.  File timestamps are ignored, allowing regenerated files to
    be properly uploaded only if their contents have changed.

    Call `setHost` and `setMd5File` to establish the settings for a session,
    then `upload` for each set of files to upload.  If you want to have
    removed local files automatically delete the remote files, call
    `deleteOldFiles` once, then `finish` to perform the closing bookkeeping.

    ::

        fu = FtpsUpload(config, 'ftp.myhost.com', 'myusername', 'password')
        fu.setHost('ftp.myhost.com', 'myusername', 'password')
        fu.setMd5File('myhost.md5') # optional
        fu.upload(
            hostdir='www', src='.',
            text='*.html *.css', binary='*.gif *.jpg'
        )
        # more upload() calls can go here..
        fu.deleteOldFiles()
        fu.finish()

    """

    def __init__(self, host, username, password, id="master"):
        upload.Upload.__init__(self, host, username, password, id)

        self.ftp = None
        self.ezftp = None
        self.md5file = None
        self.md5DictIn = {}
        self.md5DictOut = {}
        self.md5DictUp = {}
        # self.setHost(host, username, password)

    def setHost(self, host, username, password):
        """
        Set the host, the username and password.
        """
        if not self.ftp:
            try:
                hoststr, portstr = host.split(':')
            except:
                hoststr = host
                portstr = None
            self.ftp = tyFTP(timeout=60)
            self.ftp.set_debuglevel(3)
            if portstr:
                port = int(portstr)
                self.ftp.connect(hoststr, port)
            else:
                self.ftp.connect(hoststr, 22)
            # switch to secure data connection.. IMPORTANT! Otherwise, only the user and password is encrypted and not all the file data.
            self.ftp.auth()
            self.ftp.login(username, password)
            self.ftp.prot_p()
            self.ftp.set_pasv(1)



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import os
import psutil

import introspection
import thread
from collections import OrderedDict
import tempfile
import subprocess
import datetime
import logging
import re
import sys

import busIndipendentMessages

import configInstance
import computation_job_inputs
import output_parameters
import resource_cleaner
import resource_monitor


class ServiceBot(object):

    """This script is the remote WPS agent. One instance of this agent runs on each computational node connected to the WPS for each algorithm available. The script runs continuosly.
    """
    def __init__(self, remote_config_filepath, service_config_filepath):

        #read remote config file
        self._remote_config_filepath = remote_config_filepath
        remote_config = configInstance.create(self._remote_config_filepath)
        bus_class_name = remote_config.get("DEFAULT", "bus_class_name") #identify the class implementation of the cominication bus
        self._resource_file_dir = remote_config.get_path("DEFAULT", "resource_file_dir") # directory used to store file for resource cleaner
        if remote_config.has_option("DEFAULT", "wps_execution_shared_dir"):
            self._wps_execution_shared_dir = remote_config.get_path("DEFAULT", "wps_execution_shared_dir") # directory used to store the process encoded outputs (usually on a shared fs)

            #ensure outdir exists
            if not self._wps_execution_shared_dir.exists():
                self._wps_execution_shared_dir.mkdir()
        else:
            self._wps_execution_shared_dir = None

        #read service config, with raw=true that is without config file's value interpolation. Interpolation values are prodice only for the process bot (request hanlder); for example the unique execution id value to craete the sand box directory
        self._service_config_file = service_config_filepath
        serviceConfig = configInstance.create(service_config_filepath, case_sensitive=True, variables = {'wps_execution_shared_dir' : self._wps_execution_shared_dir}, raw=True)
        self.service = serviceConfig.get("DEFAULT", "service") #WPS service name?
        self.namespace = serviceConfig.get("DEFAULT", "namespace")
        self.description = serviceConfig.get("DEFAULT", "description") #WPS service description
        self._active = serviceConfig.get("DEFAULT", "active").lower() == "true" #True
        self._output_dir =  serviceConfig.get_path("DEFAULT", "output_dir")
        self._max_running_time = datetime.timedelta( seconds = serviceConfig.getint("DEFAULT", "max_running_time_seconds") )

        try:
            import json
            self._process_blacklist = json.loads(serviceConfig.get("DEFAULT", "process_blacklist"))
        except:
            self._process_blacklist = []

        input_sections = OrderedDict()
        for input_section in [s for s in serviceConfig.sections() if 'input' in s.lower() or 'const' in s.lower()]:
            #service bot doesn't have yet the execution unique id, thus the serviceConfig is read with raw=True to avoid config file variables interpolation
            input_sections[input_section] = serviceConfig.items_without_defaults(input_section, raw=True)
        self._input_parameters_defs = computation_job_inputs.ComputationJobInputs.create_from_config( input_sections )

        output_sections=OrderedDict()
        for output_section in [s for s in serviceConfig.sections() if 'output' in s.lower()]:
            output_sections[output_section] = serviceConfig.items_without_defaults(output_section, raw=True)
        self._output_parameters_defs = output_parameters.OutputParameters.create_from_config( output_sections, self._wps_execution_shared_dir )

        #create the concrete bus object
        self.bus = introspection.get_class_three_arg(bus_class_name, remote_config, self.service, self.namespace)

        self.bus.RegisterMessageCallback(busIndipendentMessages.InviteMessage, self.handle_invite)
        self.bus.RegisterMessageCallback(busIndipendentMessages.ExecuteMessage, self.handle_execute)

        # -- Register here the callback to the "getloadavg" message
        self.bus.RegisterMessageCallback(busIndipendentMessages.GetLoadAverageMessage, self.handle_getloadavg)

        #self._lock_running_process =  thread.allocate_lock() #critical section to access running_process from separate threads
        self.running_process={}

        self._redirect_process_stdout_to_logger = True #send the process bot (aka request handler) stdout to service bot (remote wps agent) log file
        self._remote_wps_endpoint = None

        # Allocate and start a Resource Monitoring Thread
        try:
            load_average_scan_minutes = serviceConfig.getint("DEFAULT", "load_average_scan_minutes")
        except:
            load_average_scan_minutes = 15
        self._resource_monitor = resource_monitor.ResourceMonitor( load_average_scan_minutes )
        self._resource_monitor.start()

    def get_resource_file_dir(self):
        return self._resource_file_dir

    def get_wps_execution_shared_dir(self):
        return self._wps_execution_shared_dir

    def max_execution_time(self):
        return self._max_running_time

    def run(self):
        logger = logging.getLogger("servicebot.run")
        if self._active:
            logger.info("Start listening on bus")
            self.bus.Listen()
        else:
            logger.error("This service is disabled, exit process")
            return

    def handle_invite(self, invite_message):
        """Handler for WPS invite message."""
        logger = logging.getLogger("servicebot.handle_invite")
        logger.info("handle invite message from WPS " + str(invite_message.originator()))
        if self.bus.state() != 'connected':
            try:
                self.bus.xmpp.reconnect()
                self.bus.xmpp.send_presence()
                # self.bus.xmpp.get_roster()
            except:
                logger.info( "[XMPP Disconnected]: Service "+str(self.service)+" Could not send info message to GeoServer Endpoint "+str(self._remote_wps_endpoint))
        self.bus.SendMessage(
            busIndipendentMessages.RegisterMessage(invite_message.originator(),
                                                   self.service,
                                                   self.namespace,
                                                   self.description,
                                                   self._input_parameters_defs.as_DLR_protocol(),
                                                   self._output_parameters_defs.as_DLR_protocol()
                                                   )
        )

    def handle_execute(self, execute_message):
        """Handler for WPS execute message."""
        logger = logging.getLogger("servicebot.handle_execute")

        #save execute messsage to tmp file to enable the process bot to read the inputs
        tmp_file = tempfile.NamedTemporaryFile(prefix='wps_params_', suffix=".tmp", delete=False)
        execute_message.serialize( tmp_file )
        param_filepath = tmp_file.name
        tmp_file.close()
        logger.debug("save parameters file for executing process " + self.service+  " in " + param_filepath)

        #create the Resource Cleaner file containing the process info. The "invoked_process.pid" will be set by the spawned process itself
        try:
            r = resource_cleaner.Resource()
            #create a resource...
            r.set_from_servicebot(execute_message.UniqueId(), self._output_dir / execute_message.UniqueId())
            #... and save to file
            logger.info("Start the resource cleaner!")
            r.write()
        except Exception as ex:
            logger.exception("Resource Cleaner initialization error")

        #invoke the process bot (aka request handler) asynchronously
        cmd = 'python wpsagent.py -r ' + self._remote_config_filepath + ' -s ' + self._service_config_file + ' -p ' + param_filepath + ' process'
        invoked_process = subprocess.Popen(args=cmd.split(), stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        logger.info("created process " + self.service +  " with PId " + str(invoked_process.pid) + " and cmd: " + cmd )

        #use a parallel thread to wait the end of the request handler process and get the exit code of the just created asynchronous process computation
        thread.start_new_thread(self.output_parser_verbose, (invoked_process, param_filepath,))

        logger.info("end of execute message handler, going back in listening mode")

    def handle_getloadavg(self, getloadavg_message):
        """Handler for WPS 'getloadavg' message."""
        logger = logging.getLogger("servicebot.handle_getloadavg")
        logger.info("handle getloadavg message from WPS " + str(getloadavg_message.originator()))
        # Collect current Machine Load Average and Available Memory info

        try:
            loadavg = self._resource_monitor.cpu_perc[0]
            vmem    = self._resource_monitor.vmem_perc[0]

            logger.info("Scanning Running Process. Declared Black List: %s" % self._process_blacklist)
            if self._resource_monitor.proc_is_running(self._process_blacklist) == True:
                logger.info("A process listed in blacklist is running! Setting loadavg and vmem to (100.0, 100.0)")
                loadavg = 100.0
                vmem    = 100.0
            else:
                logger.info("No blacklisted process was found. Setting loadavg and vmem to (%s, %s)" % (loadavg, vmem))

            outputs = dict()
            outputs['loadavg'] = [loadavg, 'Average Load on CPUs during the last 15 minutes.']
            outputs['vmem']    = [vmem, 'Percentage of Memory used by the server.']

            # Send the message back to the WPS
            if self.bus.state() != 'connected':
                try:
                    self.bus.xmpp.reconnect()
                    self.bus.xmpp.send_presence()
                    # self.bus.xmpp.get_roster()
                except:
                    logger.info( "[XMPP Disconnected]: Service "+str(self.service)+" Could not send info message to GeoServer Endpoint "+str(self._remote_wps_endpoint))
            self.bus.SendMessage(
                busIndipendentMessages.LoadAverageMessage(
                    getloadavg_message.originator(),
                    outputs
                    )
                )
        except Exception as ex:
            logger.exception("Load Average initialization error")

    #def output_parser(self, invoked_process):
    #    #silently wait the end of the computaion
    #    logger = logging.getLogger("servicebot.output_parser")
    #    return_code = invoked_process.wait()
    #    logger.info("Process " + self.service +  " PId " + str(invoked_process.pid)  + " terminated with exit code " + str(return_code) )
    #    if return_code != 0:
    #        logger.critical("Process " + self.service +  " PId " + str(invoked_process.pid)  + " terminated with exit code " + str(return_code))

    def output_parser_verbose(self, invoked_process, param_filepath):
        logger = logging.getLogger("servicebot.output_parser_verbose")
        logger.info("wait for end of execution of created process " + self.service +  ", PId " + str(invoked_process.pid) )

        gs_UID = None
        gs_JID = None
        gs_MSG = None
        while True:
            line = invoked_process.stdout.readline()
            if line != '':

                # Look for GeoServer JID from Process
                gs_UID_search = re.search('<UID>(.*)</UID>', line, re.IGNORECASE)
                gs_JID_search = re.search('<JID>(.*)</JID>', line, re.IGNORECASE)
                if gs_UID_search:
                    try:
                        gs_UID = gs_UID_search.group(1)
                        gs_JID = gs_JID_search.group(1)

                        gs_MSG = gs_JID_search = re.search('<MSG>(.*)</MSG>', line, re.IGNORECASE).group(1)
                    except:
                        pass

                if self._redirect_process_stdout_to_logger:
                    line = line.strip()
                    logger.debug("[SERVICE] " + line)
            else:
                logger.debug("created process " + self.service +  ", PId " + str(invoked_process.pid) + " stopped send data on stdout")
                break #end of stream

        #wait for process exit code
        return_code = invoked_process.wait()
        if return_code != 0:
            msg = "Process " + self.service +  " PId " + str(invoked_process.pid)  + " terminated with exit code " + str(return_code)
            logger.critical(msg)

            logger.debug("gs_UID[%s] / gs_JID[%s]" % (gs_UID, gs_JID))
            if gs_UID and gs_JID:
                self.bus.SendMessage( busIndipendentMessages.ErrorMessage( gs_JID, msg + " Exception: " + str(gs_MSG), gs_UID ) )
            elif self._remote_wps_endpoint:
                self.bus.SendMessage( busIndipendentMessages.ErrorMessage( self._remote_wps_endpoint, msg ) )
            else:
                exe_msg = None
                try:
                    logger.debug("Trying to recover Originator from Process Params!")
                    exe_msg = busIndipendentMessages.ExecuteMessage.deserialize( param_filepath )
                    if exe_msg.originator():
                        self.bus.SendMessage( busIndipendentMessages.ErrorMessage( exe_msg.originator(), msg + " Exception: remote process exception. Please check outputs!", exe_msg.UniqueId() ) )
                except:
                    pass
                if not exe_msg:
                    msg = "Process " + self.service +  " PId " + str(invoked_process.pid)  + " STALLED! Don't know who to send ERROR Message..."
                    logger.error(msg)
        else:
            msg = "Process " + self.service +  " PId " + str(invoked_process.pid)  + " terminated successfully!"
            logger.debug(msg)

    def send_error_message(self, msg):
        logger = logging.getLogger("ServiceBot.send_error_message")
        logger.error( msg )
        if self.bus.state() != 'connected':
            try:
                self.bus.xmpp.reconnect()
                self.bus.xmpp.send_presence()
                # self.bus.xmpp.get_roster()
            except:
                logger.info( "[XMPP Disconnected]: Service "+str(self.service)+" Could not send error message to GeoServer Endpoint "+str(self._remote_wps_endpoint))
        if self._remote_wps_endpoint:
            self.bus.SendMessage( busIndipendentMessages.ErrorMessage(  self._remote_wps_endpoint, msg ) )
        else:
            msg = "Process " + str(self.service) +  " STALLED! Don't know who to send ERROR Message..."
            logger.error(msg)

    def disconnect(self):
        self.bus.disconnect()



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import os, sys, inspect, urllib, json
import logging
import traceback

os.chdir(os.path.join(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))))
sys.path.insert(0, os.getcwd())
sys.path.append(os.path.abspath('lib'))

import sleekxmpp
from sleekxmpp.exceptions import IqError, IqTimeout

class XMPPPresenceMessage(object):

    def __init__(self, xmppChannel):
        self.xmppChannel=xmppChannel

    def send(self):
        self.xmppChannel.send_presence()
        self.xmppChannel.JoinmMUC()

class XMPPRegisterMessage(object):

    def __init__(self, xmppChannel, originator, service, namespace, descritpion, par, output):
        self.xmppChannel=xmppChannel
        self.originator=originator
        self.service=service
        self.namespace=namespace
        self.description=descritpion
        self._input_parameter=par
        self.output=output

    def title(self):
        return self.namespace + "." + self.service 

    def send(self):
        service = {}
        service['title'] = self.title()
        service['namespace'] = self.namespace
        service['description'] = self.description
        #process_buffer & result_size kept for back compatibility with STBGIS protocol 
        service['process_buffer'] = 0
        service['result_size'] = 0
        service['input']= self._input_parameter
        service['output']= self.output
        body = ''.join(['topic=register', '&service=', self.xmppChannel.get_fully_qualified_service_name(), '&message=', urllib.quote(json.dumps(service))])

        #self.xmppChannel.xmpp.send_message(mto=self.originator, mbody=body, mtype='chat')
        self.xmppChannel.xmpp.send_message(mto=self.originator, mbody=body, mtype='chat')

class XMPPProgressMessage(object):

    def __init__(self, originator, xmppChannel, progress):
        self.xmppChannel=xmppChannel
        self.progress=progress
        self.originator=originator

    def send(self):
        body = ''.join(['topic=progress', '&id=', self.xmppChannel.id, '&message=', str( self.progress)])
        self.xmppChannel.xmpp.send_message(mto=self.originator, mbody=body, mtype='chat')

class XMPPLogMessage(object):

    def __init__(self, originator, xmppChannel, level, msg):
        self.xmppChannel=xmppChannel
        self.level=level
        self.msg=msg
        self.originator=originator

    def send(self):
        body = ''.join(['topic=log', '&id=', self.xmppChannel.id, '&level=', self.level, "&message=", self.msg ])
        self.xmppChannel.xmpp.send_message(mto=self.originator, mbody=body, mtype='chat')

class XMPPCompletedMessage(object):

    def __init__(self, originator, xmppChannel, base_url, outputs):
        self.xmppChannel = xmppChannel
        self.originator = originator
        
        self._base_url = base_url
        self._outputs = outputs

    def send(self):
        try:
            message = ""
            for out_param_name, out_param_values in self._outputs.items():
                result = dict()
                result[out_param_name+'_value']         = out_param_values[0] #value
                result[out_param_name+'_description']   = out_param_values[1] #description
                result[out_param_name+'_title']         = out_param_values[2] #title
                result[out_param_name+'_type']          = out_param_values[3] #type
                result[out_param_name+'_pub']           = out_param_values[4] #publish as layer
                result[out_param_name+'_layer_name']    = out_param_values[5] #publish layer name
                result[out_param_name+'_style']         = out_param_values[6] #publish default Style
                result[out_param_name+'_workspace']     = out_param_values[7] #publish target Workspace
                result[out_param_name+'_metadata']      = out_param_values[8] #publish target MetaData
                result_json = json.dumps(result)
                result_json_url_enc=urllib.quote(result_json)
                if len(message) > 0:
                    message = message + "&"
                message = message + out_param_name + "=" + result_json_url_enc

            body = ''.join(['topic=completed', '&id=', self.xmppChannel.id,"&baseURL=",self._base_url, "&message=completed","&",message])
            self.xmppChannel.xmpp.send_message(mto=self.originator, mbody=body, mtype='chat')
        except Exception, err:
            body = ''.join(['topic=error', '&id=', self.xmppChannel.id, "&message=", "Critical error while encoding outuput!"])
            self.xmppChannel.xmpp.send_message(mto=self.originator, mbody=body, mtype='chat')
            logging.exception(str(traceback.format_exc()))
            raise

class XMPPErrorMessage(object):

    def __init__(self, originator, xmppChannel, msg, id=None):
        self.originator = originator
        self.xmppChannel = xmppChannel
        self.msg  = msg
        if id:
            self.id = id
        else:
            self.id = self.xmppChannel.id

    def send(self):

        error_json = json.dumps(self.msg)
        error_json_url_enc=urllib.quote(error_json)

        body = ''.join(['topic=error', '&id=', self.id, "&message=", error_json_url_enc])
        self.xmppChannel.xmpp.send_message(mto=self.originator, mbody=body, mtype='chat')

class XMPPLoadAverageMessage(object):

    def __init__(self, originator, xmppChannel, outputs):
        self.xmppChannel = xmppChannel
        self.originator = originator
        self._outputs = outputs

    def send(self):
        try:
            message = ""
            for out_param_name, out_param_values in self._outputs.items():
                result = dict()
                #print str(out_param_values)
                result[out_param_name+'_value']         = out_param_values[0] #value
                result[out_param_name+'_description']   = out_param_values[1] #description
                result_json = json.dumps(result)
                result_json_url_enc=urllib.quote(result_json)
                if len(message) > 0:
                    message = message + "&"
                message = message + "result_" + out_param_name + "=" + result_json_url_enc

            body = ''.join(['topic=loadavg', '&id=', self.xmppChannel.id, "&message=loadavg","&",message])
            self.xmppChannel.xmpp.send_message(mto=self.originator, mbody=body, mtype='chat')
        except Exception, err:
            print traceback.format_exc()
            body = ''.join(['topic=log', '&id=', self.xmppChannel.id, '&level=warning', "&message=", "Critical error while encoding LoadAverageMessage outuputs!"])
            self.xmppChannel.xmpp.send_message(mto=self.originator, mbody=body, mtype='chat')




import os
import psutil
import time
 
logPath = r'C:\tmp\processlogs'
if not os.path.exists(logPath):
    os.mkdir(logPath)
 
separator = "-" * 150
format  = "%7s %7s %7s %7s %7s %12s %12s %30s %30s, %s"
format2 = "%7s %7s %7s %7.4f %7.2f %12s %12s %30s %30s, %s"
while 1:
    logPath = r'C:\tmp\processlogs\procLog%i.log' % int(time.time())
    f = open(logPath, 'w')
    f.write(separator + "\n")
    f.write(time.ctime() + "\n")
    f.write(format % ("PID", "PPID", "STATUS", "%CPU", "%MEM", "VMS", "RSS", "USER", "NAME", "PATH"))
    f.write("\n")
 
    for proc in psutil.process_iter(): 
        
        process = psutil.Process(proc.pid).as_dict() # Get the process info using PID

        pid    = str(process["pid"])
        ppid   = str(process["ppid"])
        status = process["status"]

        cpu_percent = process["cpu_percent"]
        mem_percent = process["memory_percent"]
        
        rss = str(process["memory_info"].rss)
        vms = str(process["memory_info"].vms)
        username = process["username"]
        name = process["name"] # Here is the process name
        path = process["cwd"]
        f.write(format2 % (pid, ppid, status, cpu_percent, mem_percent, vms, rss, username, name, path))
        f.write("\n\n")
    f.close()
    print "Finished log update!"
    time.sleep(300)
    print "writing new log data!"


# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

#[Action8]
#class = updateINIfile
#input_ref = timeHorizon
#source_filepath = wcsOAAonDemand_template.properties
#target_filepath= %(workdir)s\wcsOAAonDemand.properties
#section = DEFAULT
#alias = time_horizont_days

import ConfigParser
import path

import computational_job_input_action

class ComputationalJobInputActionUpdateINIFile(computational_job_input_action.ComputationalJobInputAction):


    def __init__(self, input_ref, source, target, section, alias=None):
        super(ComputationalJobInputActionUpdateINIFile, self).__init__()
        #if not type(input_refs) is list:
        #    self._input_refs = [input_refs]
        #else:
        self._input_ref = input_ref
        self._source = source if isinstance(source, path.path) else path.path(source)
        self._target = target if isinstance(target, path.path) else path.path(target)
        self._section = section
        self._alias = alias
        self._value = None


    def set_inputs(self, inputs):
        if self._input_ref in inputs.names():
            self._value = inputs[self._input_ref].get_value()
            if not self.exists() and self._source != None:
                self._source.copyfile( self._target )
                self.update_file(inputs)
            elif not self.exists() and self._source == None:
                raise Exception("Cannot find target INI file " + str(self._target) )
            else:
                self.update_file(inputs)

    def update_file(self, inputs):
        srcINI = ConfigParser.RawConfigParser()
        #read
        src = self._target.open()
        srcINI.readfp(src)
        src.close()

        #update
        srcINI.set(self._section, self.get_attribute_name(), self._value)
                
        #write
        trg = self._target.open('w')
        srcINI.write( trg )
        trg.close()

    def get_attribute_name(self):
        return self._input_ref if self._alias==None else self._alias

    def exists(self):
        return self._target.exists()


class ComputationalJobInputActionUpdateINIFileAsList(ComputationalJobInputActionUpdateINIFile):


    def __init__(self, input_ref, source, target, section, alias=None):
        super(ComputationalJobInputActionUpdateINIFileAsList, self).__init__(input_ref, source, target, section, alias)



    def update_file(self, inputs):
        srcINI = ConfigParser.RawConfigParser()
        #read
        src = self._target.open()
        srcINI.readfp(src)
        src.close()

        json_list_str = ",".join( [ '"' + str(token) + '"' for token in self._value.split(",")] )

        #update
        srcINI.set(self._section, self.get_attribute_name(), "[" + json_list_str + "]")
                
        #write
        trg = self._target.open('w')
        srcINI.write( trg )
        trg.close()




# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import configInstance
import os
import path
import unittest
import datetime
import ConfigParser
import psutil
import json
import time
import logging
import traceback


class Resource(object):
    """
    Identify a process generated by a remote WPS call.
    Every processbot (aka request handler) create a resource file in pid_files_dir once started. 
    The method Resource.cleanup() looks fore pending resource files and if found might decide to kill process id found inside the file.
    """
    sleep_time_seconds = 3600
    process_time_threshold = None
    workdir_time_threshold = None
    pid_files_dir = None #dir where file for each resource is stored
    
    @staticmethod
    def get_pid_files_dir():
        return Resource.pid_files_dir if isinstance(Resource.pid_files_dir, path.path) else path.path(Resource.pid_files_dir)

    @staticmethod
    def clean_up_all():
        """Checks all resource files .pid and execute clean up"""
        logger = logging.getLogger("Resource.clean_up_all")
        logger.info("start resource cleaner main loop")
        while True:
            Resource.clean_up_iteration()

    @staticmethod
    def clean_up_iteration():
        logger = logging.getLogger("Resource.clean_up")
        logger.debug("Sleep for " +  str(Resource.sleep_time_seconds.total_seconds()) + " seconds")
        time.sleep(Resource.sleep_time_seconds.total_seconds())
        logger.debug("look for resources to clean up")
        Resource.pid_files_dir = Resource.get_pid_files_dir()
        for f in Resource.pid_files_dir.listdir("*.pid"):
            logger.debug("found resource file " + str(f))
            r = Resource()
            r.read_from_file(f)
            r.clean_up()

    @staticmethod
    def create_from_file(unique_id, process_bot_pid=None):
        r = Resource()
        if process_bot_pid ==  None:
            r.set_processbot_pid(os.getpid()) 
        else:
            r.set_processbot_pid(process_bot_pid) 
        r.set_unique_id(unique_id) 
        r.read()
        return r

    def __init__(self):
        self._start_time = None #process start time
        self._processbot_cmd_line = None #processbot (aka request handler) cmd line
        self._unique_id = None #processbot (aka request handler) unique execution id (used to crate sand box dir)
        self._sandbox_path = None #processbot (aka request handler) sandbox path
        self._processbot_pid = None #processbot (aka request handler) Process id
        self._spawned_process_pids = None #processbot (aka request handler) spawned processes ids
        self._spawned_process_cmd = None #processbot (aka request handler) spawned processes cmd lines
        self._path =  Resource.get_pid_files_dir() #dir where file for each resource is stored

    def start_time(self):
        return self._start_time

    def start_time_as_str(self):
        return self._start_time.strftime("%Y-%m-%dT%H:%M:%S")

    def cmd_line(self):
        return self._processbot_cmd_line

    def unique_id(self):
        return self._unique_id 

    def set_unique_id(self, unique_id):
        self._unique_id = unique_id

    def sendbox_path(self):
        return self._sandbox_path

    def processbot_pid(self):
        return self._processbot_pid

    def set_processbot_pid(self, pid):
        self._processbot_pid = int(pid)

    def spawned_process_pids(self):
        return self._spawned_process_pids 

    def spawned_process_cmd(self):
        return self._spawned_process_cmd

    def set_from_servicebot(self, unique_id, sendbox_path):
        self._start_time = self.now() 
        self._unique_id = unique_id
        self._sandbox_path = sendbox_path
    
    def set_from_processbot(self, processbot_pid, spawned_process_pids):
        """create the resource except for the information realted to the processbot spawned processes"""
        #if not processbot_pid in psutil.pids():
        if not psutil.pid_exists(processbot_pid):
            raise Exception("cannot find process bot pid + " + str(processbot_pid))
        proc = psutil.Process( processbot_pid )
        try:
            self._processbot_cmd_line = ' '.join(proc.cmdline())
        except:
            self._processbot_cmd_line = ' '.join(proc.cmdline)
        self._processbot_pid = processbot_pid

        """add the processbot spawned processes pids and cmdline to the resource"""
        self._spawned_process_pids=spawned_process_pids
        self._spawned_process_cmd = []
        removed=[]
        for pid in self._spawned_process_pids:
            pid = int(pid)
            #if pid is dead remove it from the list
            #if not pid in psutil.pids():
            if not psutil.pid_exists(pid):
                removed.append( pid )
            else:
                try:
                    #here pid *should* exists, however get cmd line in try, except block
                    proc = psutil.Process( pid )
                    try:
                        self._spawned_process_cmd.append( ' '.join(proc.cmdline()) )
                    except:
                        self._spawned_process_cmd.append( ' '.join(proc.cmdline) )
                except:
                    removed.append( pid )
        self._spawned_process_pids = list(set(self._spawned_process_pids) - set(removed))

    def now(self):
        return datetime.datetime.now()

    def filepath(self):
        #return self._path / str(self._processbot_pid) + "__" + str(self._unique_id) + ".pid"
        return self._path / str(self._unique_id) + ".pid"

    def read(self):
        if self.filepath().exists():
            self.read_from_file(  self.filepath() )

    def read_from_file(self, filepath):
        config = configInstance.create( filepath, raw= True) #todo: use file lock
        self._start_time = config.get("DEFAULT", "start_time")
        try: 
            self._start_time = datetime.datetime.strptime( self._start_time, "%Y-%m-%dT%H:%M:%S" )
        except:
            self._start_time = datetime.datetime.strptime( "2000-01-01T00:00:00", "%Y-%m-%dT%H:%M:%S" )

        self._processbot_cmd_line = config.get("DEFAULT", "cmd_line")
        if self._processbot_cmd_line != None and self._processbot_cmd_line !='':
            self._processbot_cmd_line = json.loads(self._processbot_cmd_line)

        self._unique_id = config.get("DEFAULT", "unique_id")
        self._sandbox_path = config.get_path("DEFAULT", "sendbox_path")
        try:
            self._processbot_pid = config.getint("DEFAULT", "processbot_pid")
        except ValueError:
            self._processbot_pid = config.get("DEFAULT", "processbot_pid")

        self._spawned_process_pids = config.get("DEFAULT", "spawned_process_pids")
        if self._spawned_process_pids != None and self._spawned_process_pids !='':
            self._spawned_process_pids = json.loads( self._spawned_process_pids )
        
        self._spawned_process_cmd = config.get("DEFAULT", "spawned_process_cmd")
        if self._spawned_process_cmd != None and self._spawned_process_cmd !='':
            self._spawned_process_cmd = json.loads(self._spawned_process_cmd)
    
    def write(self):
        logger = logging.getLogger("Resource.write")

        if self.filepath().exists():
            self.filepath().remove()
        
        config = ConfigParser.ConfigParser()

        config.set("DEFAULT", "start_time",  self.start_time_as_str() if self._start_time != None else "") 
        config.set("DEFAULT", "unique_id", self._unique_id )
        config.set("DEFAULT", "sendbox_path", str(self._sandbox_path) )

        if self._processbot_pid != None:
            config.set("DEFAULT", "processbot_pid", str(self._processbot_pid) ) 
        else:
            config.set("DEFAULT", "processbot_pid", "" ) 

        if self._processbot_cmd_line != None:
            config.set("DEFAULT", "cmd_line", json.dumps(self._processbot_cmd_line) ) 
        else:
            config.set("DEFAULT", "cmd_line", "" ) 

        if self._spawned_process_pids != None:
            config.set("DEFAULT", "spawned_process_pids", json.dumps(self._spawned_process_pids) ) 
        else:
            config.set("DEFAULT", "spawned_process_pids", "" ) 

        if self._spawned_process_cmd != None:
            config.set("DEFAULT", "spawned_process_cmd", json.dumps(self._spawned_process_cmd) ) 
        else:
            config.set("DEFAULT", "spawned_process_cmd", "" ) 

        try:
            fp = self.filepath().open('wb') #todo: use file lock
            config.write(fp)
            fp.close()
        except Exception as ex:
            logger.error("could not create file '"+self.filepath()+"' due to: " + str(ex))
            raise ex("could not create file '"+self.filepath()+"'")

    def kill_spawned_process(self):
        """kill all spawned process generated by processbot"""
        res = []
        logger = logging.getLogger("Resource.kill_spawned_process")
        logger.info("Try to kill computational job process(es)")
        for p,c in zip(self._spawned_process_pids, self._spawned_process_cmd):
            res.append ( self.kill_process(p, c) )
        res = list(set(res))
        return len(res)==0 or (len(res)==1 and res[0]) #all success

    def kill_processbot(self):
        return self.kill_process(self._processbot_pid, self._processbot_cmd_line)

    def kill_process(self, pid, cmd_line):
        logger = logging.getLogger("Resource.kill_process")
        if pid:
            try:
                logger.info("request for killing process " + str(pid))
                pid = int(pid)
                #if pid in psutil.pids():
                if psutil.pid_exists(pid):
                    logger.info("process " + str(pid) + " exists try to kill...")
                    proc = psutil.Process( pid )
                    if proc.cmdline() == cmd_line:
                        try:
                            proc.kill()
                            logger.info("process " + str(pid) + " killed")
                            return True
                        except Exception as ex:
                            logger.warning("Failure trying to kill process " + str(pid) + " due to: " + str(ex))
                            logger.debug( traceback.format_exc() )
                            return False
                    else:
                        logger.info("pid " + str(pid) + " found but with wrong command line. Assuming resource was already correcly cleaned-up")
                    return True
                else:
                    logger.warning("pid " + str(pid) + " doesn't exist")
                    return True
            except:
                logger.info("could not kill process " + str(pid))

    def delete_sandbox_dir(self):
        logger = logging.getLogger("Resource.delete_sandbox_dir")
        if self._sandbox_path.exists():
            logger.info("try to delete directory " + str(self._sandbox_path))
            try:
                self._sandbox_path.rmtree()
                logger.info("Directory " + str(self._sandbox_path) + " correctly deleted")
                return True
            except Exception as ex:
                logger.warning("Failure in deleting directory " + str(self._sandbox_path) + " due to: " + str(ex))
                logger.debug( traceback.format_exc() )
                return False

        else:
            logger.debug("Directory " + str(self._sandbox_path) + " doesn't exist, nothing to delete")
            return True

    def clean_up(self):
        logger = logging.getLogger("Resource.clean_up")
        now = self.now() 
        dt = now - self._start_time 
        if dt > Resource.process_time_threshold:
            logger.info("start to clean up resource: " + str(self) )    
            res=[]
            res.append( self.kill_spawned_process() )
            res.append( self.kill_processbot() )
            res.append( self.delete_sandbox_dir() )
            
            logger.debug( "kill computational job, kill request handler process,  delete sandbox dir = " + str(res))
            res = list(set(res))
            if (len(res)==1 and res[0]): #all success
                logger.info("all pending resource have been deallocated, remove resource file " + str(self.filepath()))
                self.filepath().remove()
            else:
                logger.info("Cannot deallocate all pending resource, do not remove resource file " + str(self.filepath()))
        else:
            logger.debug(str(self) + " is "+str(dt.total_seconds()) +" seconds old and it is not yet ready to be cleaned up, wait for next round")    

    def __str__(self):
        return "process " + str(self._processbot_pid) + " working on " + str(self._sandbox_path) + " from " + self.start_time_as_str()

class Test(unittest.TestCase):

    def test_workflow(self):
        
        import subprocess
        import random
        sendbox_root = path.path("./")

        Resource.sleep_time_seconds = datetime.timedelta( seconds=5)
        Resource.process_time_threshold = datetime.timedelta( seconds=1)
        Resource.workdir_time_threshold = datetime.timedelta( seconds=1)
        Resource.pid_files_dir = path.path("./")

        #srv bot
        #create process bot
        cmd = 'python -c "import time; time.sleep(80000)"'
        invoked_process = subprocess.Popen(args=cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        prcbot_pid = invoked_process.pid
        unique_id = str(random.randint(1, 1000)).zfill(5)
        sendbox_path =  sendbox_root / str(unique_id)

        r = Resource()
        r.set_from_servicebot(unique_id, sendbox_path)
        r.write()

        #proc bot
        #create computation job
        comp_job = subprocess.Popen(args=cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

        #add the pid of the computational job to the resource file
        rc = Resource.create_from_file(unique_id, prcbot_pid)
        rc.set_from_processbot( prcbot_pid, [ comp_job.pid ] ) 
        rc.write()

        #resource cleaner, should run in separate thread
        Resource.clean_up_iteration()

        self.assertFalse( r.filepath().exists() )

if __name__ == '__main__':
    unittest.main()



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

from collections import OrderedDict

import computation_job_param
import computation_job_const
class ComputationJobInputs(object):
    """Encapsulate a set of input parameters"""

    @staticmethod
    def create_from_config(input_sections):
        '''Static constructor: 
        create a ComputationJobInputs object from a input_sections: a dictionary such as { input1 : [( 'par1_input1_name' , par1_input1_value ), ( 'par2_input1_name' , par2_input1_value ), ...], input2 : [ .... ], ... }
        '''
        input_sections_reshaped=OrderedDict()
        #force the order of sections in config files: [input1], [input2], etc
        for k in sorted(input_sections):
            d=dict(input_sections[k])
            name = d['name']
            del d['name']
            input_sections_reshaped[name] = d
        return ComputationJobInputs.create_from_dict( input_sections_reshaped )

    @staticmethod
    def create_from_dict(paremeters_types_defs):
        '''Static constructor: 
        create a ComputationJobInputs object from paremeters_types_defs: {par1_name : { 'par1_attrubute1_name' : par1_attrubute1_value, ... }, par2_name : { 'par2_attrubute1_name' : par2_attrubute1_value, ... }, ... }
        '''
        cji = ComputationJobInputs()
        input_to_add = None
        for name, d in OrderedDict(paremeters_types_defs).items():
            if ('class' in d):
                if d['class'] == 'param':
                    title = d['title'] if "title" in d else None
                    default = d['default'] if "default" in d else None
                    formatter = d['formatter'] if "formatter" in d else None
                    mininum = int(d['min']) if "min" in d else 1
                    maximum = int(d['max']) if "max" in d else 1
                    input_mime_type = d['input_mime_type'] if "input_mime_type" in d else None
                    input_to_add = computation_job_param.ComputationJobParam(name, d['type'], title, d['description'], default, formatter, mininum, maximum, input_mime_type)
                elif d['class'] == 'const':
                    title = d['title'] if "title" in d else None
                    input_to_add = computation_job_const.ComputationJobConst(name, d['type'], title, d['description'], d['value'])
                else:
                    raise TypeError("Unknown class value "+ str(d['class']) + " for input " + str(name))
            else:
                raise TypeError("Cannot create computational job input without attribute class")
            cji.add_input( input_to_add )

        return cji

    def __init__(self):
        self._inputs = OrderedDict()

    def parse(self):
        pass

    def validate(self):
        pass

    def add_input(self, inputs):
        if (type(inputs) is list):
            self._inputs.update (dict( (v.get_name(), v) for v in inputs) )
        else:
            v=inputs
            self._inputs[ v.get_name() ] = v

    def set_values(self, values):
        for name, val in values.items():
            if name in self._inputs:
                self._inputs[name].set_value( val )

    def values(self):
        for v in self._inputs.itervalues():
            yield v

    def names(self):
        for k in self._inputs.iterkeys():
            yield k

    def __getitem__(self, k):
        return self._inputs[k] 

    def as_DLR_protocol(self):
        #[('result', '{"type": "string", ...ut.xml" }')]
        res = []
        for k in self._inputs.keys():
            if type(self._inputs[k]) is computation_job_const.ComputationJobConst:
                continue
            n = self._inputs[k].get_name()
            j = self._inputs[k].as_json_string()
            res.append( (n,j) )
        return res



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

#todo: one arg, two arg, etc is really ugly => find more pythoninc way

def get_class_no_arg( class_name ):
    parts = class_name.split('.')
    module = parts[0]
    m = __import__( module )
    for comp in parts[1:]:
        m = getattr(m, comp)         
    return m()

def get_class_one_arg( class_name, par1 ):
    parts = class_name.split('.')
    module = parts[0]
    m = __import__( module )
    for comp in parts[1:]:
        m = getattr(m, comp)
    return m(par1)

def get_class_two_arg( class_name, par1, par2 ):
    parts = class_name.split('.')
    module = parts[0]
    m = __import__( module )
    for comp in parts[1:]:
        m = getattr(m, comp)
    return m(par1, par2)

def get_class_three_arg( class_name, par1, par2, par3 ):
    parts = class_name.split('.')
    module = parts[0]
    m = __import__( module )
    for comp in parts[1:]:
        m = getattr(m, comp)
    return m(par1, par2, par3)

def get_class_four_arg( class_name, par1, par2, par3, par4 ):
    parts = class_name.split('.')
    module = parts[0]
    m = __import__( module )
    for comp in parts[1:]:
        m = getattr(m, comp)
    return m(par1, par2, par3, par4)


# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import string
import datetime
import json

class ComputationJobInput(object):

    def __init__(self, name, input_type, title, description, default=None, formatter=None, input_mime_type=None):
        self._name = name
        self._type = input_type
        self._default = default
        self._formatter = formatter
        self._title = title
        self._description = description
        self._input_mime_type = input_mime_type
        self._value = None
        self._value_converted = None
        self._allowed_chars = string.printable.replace(' ','')
        self._allowed_chars_url = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-._~:/?#[]@!$&'()*+,;="

    def _validate_and_convert(self, value):
        if (self._type=='int'):
            return int(value)
        elif (self._type=='float'):
            return float(value)
        elif (self._type=='string'):
            if all(c in  self._allowed_chars for c in value):
                return value
        elif (self._type=='url'):
            if all(c in  self._allowed_chars_url for c in value):
                return value
        elif self._type in ['application/json', 'application/xml']:
            return json.loads(value)
        elif (self._type=='datetime'):
            if self._formatter:
                try:
                    date_value = value.strftime(self._formatter)
                    return date_value
                except:
                    pass

            if all(c in  self._allowed_chars for c in value):
                return value

        raise TypeError("Cannot validate and convert value " + str(value) + " for type " + self._type)

    def _type_checking(self, value):
        try:
            self._validate_and_convert( value )
            return self._type
        except TypeError:
            return False

    def validate(self):
        if (not self.has_value()):
            raise TypeError("cannot find a value for parameter " + self.get_name())

        res = map(self._type_checking, self._value)
        #remove duplicates
        res = list(set(res))

        #if res has one element equal to True means all items in the list passed type checking
        if (len(res)==1 and res[0]):
            self._value_converted = map(self._validate_and_convert, self._value)
            return True
        else:
            raise TypeError("cannot validate value " + str(self._value) + " for parameter " + self.get_name() + " with type " + self._type)

    def set_value(self, value):
        if type(value) is not list:
            self._value = [value]
        else:
            self._value = value

        res = False
        try:
            res = self.validate()
            if not res:
                raise TypeError("cannot set value " + str(self._value) + " for parameter " + self.get_name() + " with type " + self._type)
        except:
            raise TypeError("cannot set value " + str(self._value) + " for parameter " + self.get_name() + " with type " + self._type)

    def get_value(self):
        if type(self._value_converted) is list and len(self._value_converted)==1:
            return self._value_converted[0]
        else:
            return self._value_converted

    def get_type(self):
        return self._type

    def get_input_mime_type(self):
        return self._input_mime_type

    def get_value_string(self):
        if type(self._value) is list and len(self._value)==1:
            if type(self._value[0]) is datetime.datetime:
                return self._value[0].strftime(self._formatter)
            else:
                return self._value[0]
        else:
            return self._value

    def get_value_as_JSON_literal(self):
        res = map(self._JSON_token_converter, self._value_converted)
        if type(res) is list and len(res)==1:
            return res[0]
        else:
            return res

    def _JSON_token_converter(self, value):
        if self._type=='int' or self._type=='float':
            return str(value)
        elif self._type=='string':
            return "'" + str(value) + "'"
        elif self._type=='datetime':
            return "'" +  value.strftime(self._formatter) + "'"
        elif self._type=='application/json':
            raise TypeError("cannot convert json in json")
        else:
            raise TypeError("unknown type for value " + self.get_name() )

    def has_value(self):
        return self._value != None

    def get_name(self):
        return self._name

    def as_json_string(self):
        #{"type": "string", "description": "A persons surname", "max": 1, "default": "Meier"}
        res={}
        attrib_to_convert = ['_type', '_title', '_default', '_description', '_min', '_max', '_input_mime_type'] #missing _enum
        attribute_list = [a for a in dir(self) if not a.startswith('__') and not callable(getattr(self, a))]
        attribute_list_filtered = [x for x in attribute_list if x in attrib_to_convert]
        for a in attribute_list_filtered:
            res[a[1:]] = getattr(self, a)
        return  json.dumps(res)



# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.




# (c) 2016 Open Source Geospatial Foundation - all rights reserved
# (c) 2014 - 2015 Centre for Maritime Research and Experimentation (CMRE)
# (c) 2013 - 2014 German Aerospace Center (DLR)
# This code is licensed under the GPL 2.0 license, available at the root
# application directory.

__author__ = "Alessio Fabiani"
__copyright__ = "Copyright 2016 Open Source Geospatial Foundation - all rights reserved"
__license__ = "GPL"

import subprocess
import thread
import re
import datetime
import os
import introspection
import thread
import logging
import traceback
import base64
import sys

import busIndipendentMessages
import computation_job_inputs
import computational_job_input_actions
import configInstance
import output_parameters
import action
import resource_cleaner
from time import sleep
from collections import OrderedDict
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA


class ProcessBot(object):
    """
    This script starts when the user call a new WPS execution.
    His task is to call the proper external executable/scripts according to the service.config file (provided in the cmd line with -s option) and send back to the WPS logging/progress
    information and error information if something unexpected happens.
    All the output including the log file is generated in a sand box directory created with joint information from service.config and external process start-up information.
    """
    def __init__(self, remote_config_filepath, service_config_filepath, execute_message):
        self._uniqueExeId = execute_message.UniqueId()
        self._remote_wps_endpoint = execute_message.originator()
        self._remote_wps_baseurl = execute_message.BaseURL()
        self._input_values = execute_message.variables()

        #read remote config
        remote_config = configInstance.create(remote_config_filepath)
        bus_class_name = remote_config.get("DEFAULT", "bus_class_name")
        uploader_class_name = None
        try:
            uploader_class_name = remote_config.get("UPLOADER", "uploader_class_name")
        except:
            pass
        self._resource_file_dir = remote_config.get_path("DEFAULT", "resource_file_dir")
        if remote_config.has_option("DEFAULT", "wps_execution_shared_dir"):
            self._wps_execution_shared_dir = remote_config.get_path("DEFAULT", "wps_execution_shared_dir")

            #ensure outdir exists
            if not self._wps_execution_shared_dir.exists():
                self._wps_execution_shared_dir.mkdir()
        else:
            self._wps_execution_shared_dir = None

        #the config file is read with raw=False because the unique_exe_id value will be used (interpolated) in the config
        serviceConfig = configInstance.create(service_config_filepath, case_sensitive=True, variables = {'unique_exe_id' : self._uniqueExeId, 'wps_execution_shared_dir' : self._wps_execution_shared_dir}, raw=False)

        self.service = serviceConfig.get("DEFAULT", "service") #todo: what is?
        self.namespace = serviceConfig.get("DEFAULT", "namespace")
        self.description = serviceConfig.get("DEFAULT", "description")
        self._active = serviceConfig.get("DEFAULT", "active").lower() == "true" #True

        self._executable_path = serviceConfig.get("DEFAULT", "executable_path")
        self._executable_cmd = serviceConfig.get("DEFAULT", "executable_cmd")
        if not os.path.isabs(self._executable_path):
            full_executable_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), self._executable_path)
            self._executable_cmd = self._executable_cmd.replace(self._executable_path, full_executable_path)
            self._executable_path = full_executable_path

        self._stdout_parser = serviceConfig.get_list("Logging", "stdout_parser")
        self._stdout_action = serviceConfig.get_list("Logging", "stdout_action")
        self._output_dir =  serviceConfig.get_path("DEFAULT", "output_dir")
        if not os.path.isabs(self._output_dir):
            self._output_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), self._output_dir)
        self._max_running_time = datetime.timedelta(seconds = serviceConfig.getint("DEFAULT", "max_running_time_seconds"))

        #create the concrete uploader object
        if uploader_class_name:
            uploader_host = remote_config.get("UPLOADER", "uploader_host")
            uploader_username = remote_config.get("UPLOADER", "uploader_username")
            uploader_password = remote_config.get("UPLOADER", "uploader_password")

            if remote_config.has_option("UPLOADER", "uploader_private_rsa_key") and remote_config.has_option("UPLOADER", "uploader_passphrase"):
                uploader_private_rsa_key = remote_config.get("UPLOADER", "uploader_private_rsa_key")
                uploader_passphrase = remote_config.get("UPLOADER", "uploader_passphrase")
                privatekey = open(uploader_private_rsa_key, "r")
                rsa_key = RSA.importKey(privatekey, passphrase=uploader_passphrase)
                uploader_password = rsa_key.decrypt(base64.b64decode(uploader_password))

            self._uploader = introspection.get_class_four_arg(uploader_class_name, uploader_host, uploader_username, uploader_password, self._uniqueExeId)
        else:
            self._uploader = None

        input_sections = OrderedDict()
        for input_section in [s for s in serviceConfig.sections() if 'input' in s.lower() or 'const' in s.lower()]:
            input_sections[input_section] = serviceConfig.items_without_defaults(input_section, raw=False)
        self._input_parameters_defs = computation_job_inputs.ComputationJobInputs.create_from_config(input_sections)

        output_sections=OrderedDict()
        for output_section in [s for s in serviceConfig.sections() if 'output' in s.lower()]:
            output_sections[output_section] = serviceConfig.items_without_defaults(output_section, raw=False)
        self._output_parameters_defs = output_parameters.OutputParameters.create_from_config( output_sections, self._wps_execution_shared_dir, self._uploader)

        action_sections=OrderedDict()
        for action_section in [s for s in serviceConfig.sections() if 'action' in s.lower()]:
            action_sections[action_section] = serviceConfig.items_without_defaults(action_section, raw=False)
        self._input_params_actions = computational_job_input_actions.ComputationalJobInputActions.create_from_config(action_sections)

        #create the concrete bus object
        self._lock_bus =  thread.allocate_lock()
        self.bus = introspection.get_class_four_arg(bus_class_name, remote_config, self.service, self.namespace, self._uniqueExeId)

        self._finished = False

        #event handlers
        self.bus.RegisterMessageCallback(busIndipendentMessages.FinishMessage, self.handle_finish)
        self.bus.RegisterMessageCallback(busIndipendentMessages.AbortMessage, self.handle_abort)

    def get_resource_file_dir(self):
        return self._resource_file_dir

    def get_wps_execution_shared_dir(self):
        return self._wps_execution_shared_dir

    def max_execution_time(self):
        return self._max_running_time

    def ensure_output_dir_exists(self):
        directory = self._output_dir / self._uniqueExeId
        if (not directory.exists()):
            directory.mkdir()
        return directory

    def workdir(self):
        return self._output_dir / self._uniqueExeId

    def run(self):
        #create the computational job process asynchronously
        thread.start_new_thread(self.SpawnProcess, ())
        #start listen the bus to catch abort or finish messages
        self.bus.Listen()

    def SpawnProcess(self):
        try:
            logger = logging.getLogger("ProcessBot.SpawnProcess")
            self.ensure_output_dir_exists()

            #set the actual value of input parameters to input parameters definitions
            self._input_parameters_defs.set_values( self._input_values )

            #execute actions associated to input parameters
            self._input_params_actions.execute( self._input_parameters_defs )

            #prepare cmd line
            cmd = self._executable_cmd + " " +  self._input_params_actions.get_cmd_line()

            #spawn the computational job process
            invoked_process = subprocess.Popen(args=cmd.split(), cwd=self._executable_path, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=False)
            logger.info("process " + self.service +  " created with PId " + str(invoked_process.pid) + " and command line: " + cmd)

            #read the resource file
            rc = resource_cleaner.Resource.create_from_file(self._uniqueExeId, os.getpid())
            #add the pid of the computational job to the resource file
            rc.set_from_processbot( os.getpid(), [ invoked_process.pid ] ) #todo: check if cmds contains ","!!! --> pickle?
            rc.write()
        except Exception as ex:
            logging.exception( "Process "+str(self._uniqueExeId)+" Exception: "+str(traceback.format_exc(sys.exc_info())))
            error_message = "process failure\n" + str(ex)
            self.send_error_message( error_message )
            #self.bus.disconnect()
            logger.info( "after send job-error message to WPS")
            thread.interrupt_main()
            os._exit(return_code)

        #go to process output synchronuosly
        self.process_output_parser( invoked_process )

    def process_output_parser(self, invoked_process):
        logger = logging.getLogger("ProcessBot.process_output_parser")
        logger.info( "start parsing stdout of created process " + self.service )

        with self._lock_bus:
            if self.bus.state() == 'connected':
                self.bus.SendMessage( busIndipendentMessages.LogMessage(  self._remote_wps_endpoint, "INFO", "start parsing stdout of created process " + self.service ) )
            else:
                try:
                    self.bus.xmpp.reconnect()
                    self.bus.xmpp.send_presence()
                    # self.bus.xmpp.get_roster()

                    if self.bus.state() == 'connected':
                        self.bus.SendMessage( busIndipendentMessages.LogMessage(  self._remote_wps_endpoint, "INFO", "start parsing stdout of created process " + self.service ) )
                    else:
                        logger.info( "[XMPP Disconnected]: Process "+str(self._uniqueExeId)+" Could not send info message to GeoServer Endpoint "+str(self._remote_wps_endpoint))
                except:
                    logger.info( "[XMPP Disconnected]: Process "+str(self._uniqueExeId)+" Could not send info message to GeoServer Endpoint "+str(self._remote_wps_endpoint))

        # Listen to stdout
        stdout_parser_compiled = [re.compile(r) for r in self._stdout_parser]
        stack_trace = []
        while True:
            line = invoked_process.stdout.readline()
            if line != '':
                line = line.strip()
                logger.debug( "Received line: " + line)
                stack_trace.append(line)
                match=False
                #todo: define a line to get pid of computational job process
                for rgx,action in zip(stdout_parser_compiled, self._stdout_action):
                    res = rgx.match(line)
                    if (res):
                        if (action=="progress"):
                            with self._lock_bus:
                                if self.bus.state() != 'connected':
                                    try:
                                        self.bus.xmpp.reconnect()
                                        self.bus.xmpp.send_presence()
                                        # self.bus.xmpp.get_roster()
                                    except:
                                        logger.info( "[XMPP Disconnected]: Process "+str(self._uniqueExeId)+" Could not send info message to GeoServer Endpoint "+str(self._remote_wps_endpoint))
                                self.bus.SendMessage( busIndipendentMessages.ProgressMessage(  self._remote_wps_endpoint, float(res.group(1).strip() ) ) )
                            match=True
                            break
                        elif (action=="log"):
                            with self._lock_bus:
                                if self.bus.state() != 'connected':
                                    try:
                                        self.bus.xmpp.reconnect()
                                        self.bus.xmpp.send_presence()
                                        # self.bus.xmpp.get_roster()
                                    except:
                                        logger.info( "[XMPP Disconnected]: Process "+str(self._uniqueExeId)+" Could not send info message to GeoServer Endpoint "+str(self._remote_wps_endpoint))
                                self.bus.SendMessage( busIndipendentMessages.LogMessage(  self._remote_wps_endpoint, res.group(1).strip(), res.group(2).strip() ) )
                            match=True
                            break
                        elif (action=="abort"):
                            with self._lock_bus:
                                if self.bus.state() != 'connected':
                                    try:
                                        self.bus.xmpp.reconnect()
                                        self.bus.xmpp.send_presence()
                                        # self.bus.xmpp.get_roster()
                                    except:
                                        logger.info( "[XMPP Disconnected]: Process "+str(self._uniqueExeId)+" Could not send info message to GeoServer Endpoint "+str(self._remote_wps_endpoint))
                                self.bus.SendMessage( busIndipendentMessages.ErrorMessage(  self._remote_wps_endpoint, res.group(2).strip() ) )
                            match=True
                            break
                        elif (action=="ignore"):
                            match=True
                            break
                        else:
                            pass
            else:
                #end of stream
                break
        logger.debug( "process " + self.service + "stdout is over")
        #wait for process exit code
        return_code = invoked_process.wait()
        logger.info( "process exit code is " + str(return_code))

        if return_code==0:
            #success
            logger.info("process exit code is " + str(return_code) + ": success")
            logger.info("send job-completed message to WPS with output parameter")
            outputs = dict()
            try:
                for p in self._output_parameters_defs.parameters():
                    outputs[p.get_name()] = [p.get_value(), p.get_description(), p.get_title(), p.get_type(), p.is_publish_as_layer(), p.get_publish_layer_name(), p.get_publish_default_style(), p.get_publish_target_workspace(), p.get_metadata()]
            except:
                logging.exception( "Process "+str(self._uniqueExeId)+" Exception: "+str(traceback.format_exc(sys.exc_info())))
                error_message = "process exit code is " + str(return_code) + ": failure\n" + "\n".join(str(e) for e in stack_trace)
                self.send_error_message( error_message )
                #self.bus.disconnect()
                logger.info( "after send job-error message to WPS")
                thread.interrupt_main()
                os._exit(return_code)

            logger.info("trying to acquire bus lock...")
            with self._lock_bus:
                logger.info("bus lock acquired...")
                if self.bus.state() != 'connected':
                    try:
                        self.bus.xmpp.reconnect()
                        self.bus.xmpp.send_presence()
                        # self.bus.xmpp.get_roster()
                    except:
                        logger.info( "[XMPP Disconnected]: Process "+str(self._uniqueExeId)+" Could not send info message to GeoServer Endpoint "+str(self._remote_wps_endpoint))

                counter = 1
                while not self._finished:
                    logger.info("sending 'completed' message tentative #" + str(counter))
                    self.bus.SendMessage(busIndipendentMessages.CompletedMessage( self._remote_wps_endpoint, self._remote_wps_baseurl, outputs ))
                    counter = counter + 1
                    if counter < 100:
                        sleep(10)
                    else:
                        logger.error("Could not contact Remote WPS with. Forcibly shutdown the process..." )
                        thread.interrupt_main()
                        os._exit(-1)

            logger.info( "after send job-completed message to WPS")
        else:
            error_message = "process exit code is " + str(return_code) + ": failure\n" + "\n".join(str(e) for e in stack_trace)
            logger.critical( "process exit code is " + str(return_code) + ": failure")

            #todo: should i wait for finish message here as well? No
            self.send_error_message( error_message )

            #self.bus.disconnect()
            logger.info( "after send job-error message to WPS")
            thread.interrupt_main()
            os._exit(return_code)

    def handle_finish(self, finished_message):
        logger = logging.getLogger("ProcessBot.handle_finish")
        logger.info("received finish mesasge from WPS")
        self._finished = True
        with self._lock_bus:
            self.bus.disconnect()
        logger.info("disconnected from communication bus")

        sys.exit(0)

    def handle_abort(self, aborted_message):
        logger = logging.getLogger("ProcessBot.handle_abort")
        logger.info("received abort mesasge from WPS")
        self._finished = True
        with self._lock_bus:
            self.bus.disconnect()
        logger.info("disconnected from communication bus")

        sys.exit(-1)

    def send_error_message(self, msg):
        logger = logging.getLogger("ProcessBot.send_error_message to " + str(self._remote_wps_endpoint))
        logger.error( msg )
        with self._lock_bus:
            if self.bus.state() == 'connected':
                self.bus.SendMessage( busIndipendentMessages.ErrorMessage( self._remote_wps_endpoint, msg ) )
            else:
                try:
                    self.bus.xmpp.reconnect()
                    self.bus.xmpp.send_presence()
                    # self.bus.xmpp.get_roster()

                    if self.bus.state() == 'connected':
                        self.bus.SendMessage( busIndipendentMessages.ErrorMessage( self._remote_wps_endpoint, msg ) )
                    else:
                        sys.stdout.write( "[XMPP Disconnected]: Process <UID>"+str(self._uniqueExeId)+"</UID> Could not send error message to GeoServer Endpoint <JID>"+str(self._remote_wps_endpoint)+"</JID> <MSG>"+msg.replace('\n', ' ').replace('\r', '')+"</MSG>")
                except:
                    sys.stdout.write( "[XMPP Disconnected]: Process <UID>"+str(self._uniqueExeId)+"</UID> Could not send error message to GeoServer Endpoint <JID>"+str(self._remote_wps_endpoint)+"</JID> <MSG>"+msg.replace('\n', ' ').replace('\r', '')+"</MSG>")

        logger.debug( "send error msg complete" )
        thread.interrupt_main()
        os._exit(-1)

    def disconnect(self):
        with self._lock_bus:
            self.bus.disconnect()
